# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Enables Tkinter integration."""

__all__ = ["GlobeControl", "MapControl", "GfxAnalysisControl"]

import os
import pathlib
from tkinter import Frame

if os.name == "nt":
    from ctypes import CDLL, POINTER, WinDLL, WinError, c_char_p, cdll, create_unicode_buffer, get_last_error
else:
    from ctypes import CDLL, POINTER, cdll

from ..internal.comutil import BOOL, CHAR, DWORD, INT, LONG, LPCWSTR, LPVOID, WINFUNCTYPE, IUnknown
from ..stkengine import STKEngine
from ..stkx import Graphics2DControlBase, Graphics3DControlBase, GraphicsAnalysisControlBase
from ..utilities.exceptions import STKAttributeError, STKRuntimeError

if os.name != "nt":
    from ctypes.util import find_library

class NativeContainerMethods:
    """Provide support for STK Engine controls (map, globe, graphics analysis)."""
    def __init__(self):
        self.jniCore = CDLL(self._get_jni_core_path())
        self.AgPythonCreateContainer                                         = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPCWSTR)(("AgPythonCreateContainer", self.jniCore), ((1, "env"), (1, "_this"), (1, "progId")))
        self.Java_agi_core_awt_AgAwtNativeContainer_AttachContainer          = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LPVOID, LPVOID)(("Java_agi_core_awt_AgAwtNativeContainer_AttachContainer", self.jniCore), ((1, "env"), (1, "_this"), (1, "pNativeCanvas"), (1, "pNativeDisplay"), (1, "pContainer")))
        self.Java_agi_core_awt_AgAwtNativeContainer_ResizeContainer          = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, LONG, LONG, LONG)(("Java_agi_core_awt_AgAwtNativeContainer_ResizeContainer", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "x"), (1, "y"), (1, "width"), (1, "height")))
        self.AgPythonGetIAgUnknown                                           = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID)(("AgPythonGetIAgUnknown", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer")))
        self.Java_agi_core_awt_AgAwtNativeContainer_DetachContainer          = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID)(("Java_agi_core_awt_AgAwtNativeContainer_DetachContainer", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer")))
        self.Java_agi_core_awt_AgAwtNativeContainer_ReleaseContainer         = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID)(("Java_agi_core_awt_AgAwtNativeContainer_ReleaseContainer", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer")))
        if os.name!="nt":
            self.Java_agi_core_awt_AgAwtNativeContainer_Paint                                               = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID)(("Java_agi_core_awt_AgAwtNativeContainer_Paint", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer")))
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseAdapter_MousePressed           = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, LONG, BOOL, BOOL, BOOL, BOOL, BOOL, BOOL)(("Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseAdapter_MousePressed", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "x"), (1, "y"), (1, "leftButtonDown"), (1, "middleButtonDown"), (1, "rightButtonDown"), (1, "ctrlKeyDown"), (1, "altKeyDown"), (1, "shiftKeyDown")))
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseAdapter_MouseReleased          = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, LONG, BOOL, BOOL, BOOL, BOOL, BOOL, BOOL)(("Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseAdapter_MouseReleased", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "x"), (1, "y"), (1, "leftButtonDown"), (1, "middleButtonDown"), (1, "rightButtonDown"), (1, "ctrlKeyDown"), (1, "altKeyDown"), (1, "shiftKeyDown")))
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseMotionAdapter_MouseMoved       = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, LONG, BOOL, BOOL, BOOL, BOOL, BOOL, BOOL)(("Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseMotionAdapter_MouseMoved", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "x"), (1, "y"), (1, "leftButtonDown"), (1, "middleButtonDown"), (1, "rightButtonDown"), (1, "ctrlKeyDown"), (1, "altKeyDown"), (1, "shiftKeyDown")))
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseWheelAdapter_MouseWheelMoved   = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, LONG, LONG, BOOL, BOOL, BOOL, BOOL, BOOL, BOOL)(("Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseWheelAdapter_MouseWheelMoved", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "x"), (1, "y"), (1, "ticks"), (1, "leftButtonDown"), (1, "middleButtonDown"), (1, "rightButtonDown"), (1, "ctrlKeyDown"), (1, "altKeyDown"), (1, "shiftKeyDown")))
            self.AgPythonKeyPressed                                                                         = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, BOOL, BOOL, BOOL)(("AgPythonKeyPressed", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "keyCode"), (1, "ctrlKeyDown"), (1, "altKeyDown"), (1, "shiftKeyDown")))
            self.AgPythonKeyReleased                                                                        = WINFUNCTYPE(LPVOID, LPVOID, LPVOID, LPVOID, LONG, BOOL, BOOL, BOOL)(("AgPythonKeyReleased", self.jniCore), ((1, "env"), (1, "_this"), (1, "pContainer"), (1, "keyCode"), (1, "ctrlKeyDown"), (1, "altKeyDown"), (1, "shiftKeyDown")))
    def _get_jni_core_path(self):
        if not STKEngine._is_engine_running:
            raise STKRuntimeError("STKEngine.StartApplication() must be called before using the STK Engine controls")

        if os.name != "nt":
            return "libagjnicore.so"
        else:
            kernel32 = WinDLL("kernel32", use_last_error=True)

            kernel32.GetModuleHandleW.restype = LPVOID
            kernel32.GetModuleHandleW.argtypes = [LPCWSTR]

            stkx_module_handle = kernel32.GetModuleHandleW("stkx.dll")
            if stkx_module_handle is None:
                raise STKRuntimeError(f"Error getting stkx.dll module handle ({WinError(get_last_error())})")

            kernel32.GetModuleFileNameA.restype = DWORD
            kernel32.GetModuleFileNameA.argtypes = [LPVOID, c_char_p, DWORD]

            c_path = create_unicode_buffer(1024)
            res = kernel32.GetModuleFileNameW(LPVOID(stkx_module_handle), c_path, DWORD(1024))
            if res == 0:
                err = get_last_error()
                errormsg = "Failed to get STKX module file name"
                if err != 0:
                    errormsg += f" ({WinError(err)})"
                raise STKRuntimeError(errormsg)
            stkx_dll_path = pathlib.Path(c_path.value).resolve()

            jni_core_dll_path = stkx_dll_path.parent / "AgJNICore.dll"
            return str(jni_core_dll_path)
    def create_container(self, progid):
        return self.AgPythonCreateContainer(LPVOID(None), LPVOID(None), LPCWSTR(progid))
    def attach_container(self, container, winid, display):
        self.Java_agi_core_awt_AgAwtNativeContainer_AttachContainer(LPVOID(None), LPVOID(None), winid, display, LPVOID(container))
    def resize_container(self, container, x, y, width, height):
        self.Java_agi_core_awt_AgAwtNativeContainer_ResizeContainer(LPVOID(None), LPVOID(None), LPVOID(container), INT(x), INT(y), INT(width), INT(height))
    def get_unknown(self, container):
        return self.AgPythonGetIAgUnknown(LPVOID(None), LPVOID(None), LPVOID(container))
    def detach_container(self, container):
        self.Java_agi_core_awt_AgAwtNativeContainer_DetachContainer(LPVOID(None), LPVOID(None), LPVOID(container))
    def release_container(self, container):
        self.Java_agi_core_awt_AgAwtNativeContainer_ReleaseContainer(LPVOID(None), LPVOID(None), LPVOID(container))
    if os.name!="nt":
        def paint(self, container):
            self.Java_agi_core_awt_AgAwtNativeContainer_Paint(LPVOID(None), LPVOID(None), LPVOID(container))
        def mouse_pressed(self, container, x, y, left_button_down, middle_button_down, right_button_down, ctrl_key_down, alt_key_down, shift_key_down):
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseAdapter_MousePressed(LPVOID(None), LPVOID(None), LPVOID(container), INT(x), INT(y), BOOL(left_button_down), BOOL(middle_button_down), BOOL(right_button_down), BOOL(ctrl_key_down), BOOL(alt_key_down), BOOL(shift_key_down))
        def mouse_released(self, container, x, y, left_button_down, middle_button_down, right_button_down, ctrl_key_down, alt_key_down, shift_key_down):
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseAdapter_MouseReleased(LPVOID(None), LPVOID(None), LPVOID(container), INT(x), INT(y), BOOL(left_button_down), BOOL(middle_button_down), BOOL(right_button_down), BOOL(ctrl_key_down), BOOL(alt_key_down), BOOL(shift_key_down))
        def mouse_moved(self, container, x, y, left_button_down, middle_button_down, right_button_down, ctrl_key_down, alt_key_down, shift_key_down):
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseMotionAdapter_MouseMoved(LPVOID(None), LPVOID(None), LPVOID(container), INT(x), INT(y), BOOL(left_button_down), BOOL(middle_button_down), BOOL(right_button_down), BOOL(ctrl_key_down), BOOL(alt_key_down), BOOL(shift_key_down))
        def mouse_wheel_moved(self, container, x, y, ticks, left_button_down, middle_button_down, right_button_down, ctrl_key_down, alt_key_down, shift_key_down):
            self.Java_agi_core_awt_AgAwtNativeContainer_00024AgAwtCanvasMouseWheelAdapter_MouseWheelMoved(LPVOID(None), LPVOID(None), LPVOID(container), INT(x), INT(y), INT(ticks), BOOL(left_button_down), BOOL(middle_button_down), BOOL(right_button_down), BOOL(ctrl_key_down), BOOL(alt_key_down), BOOL(shift_key_down))
        def key_pressed(self, container, key_code, ctrl_key_down, alt_key_down, shift_key_down):
            self.AgPythonKeyPressed(LPVOID(None), LPVOID(None), LPVOID(container), INT(key_code), BOOL(ctrl_key_down), BOOL(alt_key_down), BOOL(shift_key_down))
        def key_released(self, container, key_code, ctrl_key_down, alt_key_down, shift_key_down):
            self.AgPythonKeyReleased(LPVOID(None), LPVOID(None), LPVOID(container), INT(key_code), BOOL(ctrl_key_down), BOOL(alt_key_down), BOOL(shift_key_down))

class ControlBase(Frame):
    """Base class for Tkinter controls."""

    _shift = 0x0001
    _control = 0x0004
    _lalt = 0x0008
    _ralt = 0x0080
    _mouse1 = 0x0100
    _mouse2 = 0x0200
    _mouse3 = 0x0400

    def __init__(self, parent, *args, **kwargs):
        # Set background to empty string to prevent tk from drawing background over opengl draws
        kwargs["bg"] = ""
        Frame.__init__(self, parent, *args, **kwargs)
        self._is_container_attached = False
        self._nativeContainerMethods = NativeContainerMethods()
        if os.name!="nt":
            self._x11lib = cdll.LoadLibrary(find_library("X11"))
            self._XOpenDisplay = WINFUNCTYPE(POINTER(CHAR))(("XOpenDisplay", self._x11lib))

        self._container = self._nativeContainerMethods.create_container(self._progid)
        self._unk = self._nativeContainerMethods.get_unknown(self._container)

        _cntrlinit_unk = IUnknown()
        _cntrlinit_unk.p = LPVOID(self._unk)

        self._interface._private_init(self, _cntrlinit_unk)

        self.bind("<Configure>", self._configure)
        if os.name!="nt":
            self.bind("<Expose>", self._expose)
            self.bind("<ButtonPress>", self._button_press)
            self.bind("<ButtonRelease>", self._button_release)
            self.bind("<Motion>", self._motion)
            self.bind_all("<Any-KeyPress>", self._key_press)
            self.bind_all("<Any-KeyRelease>", self._key_release)

    def __setattr__(self, attrname, value):
        try:
            self._interface.__setattr__(self, attrname, value)
        except STKAttributeError:
            Frame.__setattr__(self, attrname, value)

    def _configure(self, event):
        """Occurs when the frame is resized."""
        if not self._is_container_attached:
            self._xDisplay = None if os.name=="nt" else self._XOpenDisplay(self.winfo_screen().encode("utf-8"))
            self._nativeContainerMethods.attach_container(self._container, self.winfo_id(), self._xDisplay)
            self._is_container_attached = True
        self._nativeContainerMethods.resize_container(self._container, 0, 0, event.width, event.height)

    def destroy(self):
        """Occurs before the frame is destroyed."""
        self._nativeContainerMethods.detach_container(self._container)
        self._nativeContainerMethods.release_container(self._container)
        super().destroy()

    if os.name!="nt":
        def _expose(self, event):
            """Occurs when at least some part of the frame becomes visible after having been covered up by another window."""
            if self._is_container_attached:
                self._nativeContainerMethods.paint(self._container)

        def _button_press(self, event):
            """Occurs when a mouse button is pressed."""
            if event.num == 4:
                if not(event.state & self._mouse1 or event.state & self._mouse2 or event.state & self._mouse3):
                    self._nativeContainerMethods.mouse_wheel_moved(self._container, event.x, event.y, 1, event.num == 1, event.num == 2, event.num == 3, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)
            elif event.num == 5:
                if not(event.state & self._mouse1 or event.state & self._mouse2 or event.state & self._mouse3):
                    self._nativeContainerMethods.mouse_wheel_moved(self._container, event.x, event.y, -1, event.num == 1, event.num == 2, event.num == 3, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)
            else:
                if not(event.state & self._mouse1 or event.state & self._mouse2 or event.state & self._mouse3):
                    self._nativeContainerMethods.mouse_pressed(self._container, event.x, event.y, event.num == 1, event.num == 2, event.num == 3, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)

        def _button_release(self, event):
            """Occurs when a mouse button is released."""
            self._nativeContainerMethods.mouse_released(self._container, event.x, event.y, event.num == 1, event.num == 2, event.num == 3, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)

        def _motion(self, event):
            """Occurs when mouse motion occurs."""
            self._nativeContainerMethods.mouse_moved(self._container, event.x, event.y, event.state & self._mouse1, event.state & self._mouse2, event.state & self._mouse3, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)

        def _key_press(self, event):
            """Occurs when a key is pressed."""
            self._nativeContainerMethods.key_pressed(self._container, event.keysym_num, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)

        def _key_release(self, event):
            """Occurs when key is released."""
            self._nativeContainerMethods.key_released(self._container, event.keysym_num, event.state & self._control, event.state & self._lalt or event.state & self._ralt , event.state & self._shift)

class GlobeControl(Graphics3DControlBase, ControlBase):
    """The 3D Globe control for Tkinter."""

    _progid = "STKX12.VOControl.1"
    _interface = Graphics3DControlBase

    def __init__(self, parent, *args, **kwargs):
        """Construct an object of type GlobeControl."""
        Graphics3DControlBase.__init__(self)
        ControlBase.__init__(self, parent, *args, **kwargs)

    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        ControlBase.__setattr__(self, attrname, value)

class MapControl(Graphics2DControlBase, ControlBase):
    """The 2D Map control for Tkinter."""

    _progid = "STKX12.2DControl.1"
    _interface = Graphics2DControlBase

    def __init__(self, parent, *args, **kwargs):
        """Construct an object of type MapControl."""
        Graphics2DControlBase.__init__(self)
        ControlBase.__init__(self, parent, *args, **kwargs)

    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        ControlBase.__setattr__(self, attrname, value)

class GfxAnalysisControl(GraphicsAnalysisControlBase, ControlBase):
    """The Graphics Analysis control for Tkinter."""

    _progid = "STKX12.GfxAnalysisControl.1"
    _interface = GraphicsAnalysisControlBase

    def __init__(self, parent, *args, **kwargs):
        """Construct an object of type GfxAnalysisControl."""
        GraphicsAnalysisControlBase.__init__(self)
        ControlBase.__init__(self, parent, *args, **kwargs)

    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        ControlBase.__setattr__(self, attrname, value)


# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""The STK UI Core library is a COM library containing classes, interfaces and enumerations for the Application Object Model."""

__all__ = ["ApplicationWindowState", "Toolbar", "ToolbarCollection", "Window", "WindowArrangeState", "WindowArrangeStyle",
"WindowDockStyle", "WindowGlobeObject", "WindowMapObject", "WindowServiceType", "WindowsCollection"]

from ctypes import POINTER
from enum import IntEnum
import typing

from .internal import coclassutil as agcls, comutil as agcom, marshall as agmarshall
from .internal.apiutil import (
    EnumeratorProxy,
    InterfaceProxy,
    OutArg,
    SupportsDeleteCallback,
    get_interface_property,
    initialize_from_source_object,
    set_class_attribute,
)
from .internal.comutil import IDispatch, IUnknown
from .utilities.exceptions import STKRuntimeError


def _raise_uninitialized_error(*args):
    raise STKRuntimeError("Valid STK object model classes are returned from STK methods and should not be created independently.")

class WindowServiceType(IntEnum):
    """Well-known types of services."""

    WINDOW_2D = 1
    """A 2D window."""
    WINDOW_3D = 2
    """A 3D window."""

WindowServiceType.WINDOW_2D.__doc__ = "A 2D window."
WindowServiceType.WINDOW_3D.__doc__ = "A 3D window."

agcls.AgTypeNameMap["WindowServiceType"] = WindowServiceType

class ApplicationWindowState(IntEnum):
    """Window states."""

    MAXIMIZED = 1
    """Window is maximized."""
    MINIMIZED = 2
    """Window is minimized."""
    NORMAL = 3
    """Normal window state."""

ApplicationWindowState.MAXIMIZED.__doc__ = "Window is maximized."
ApplicationWindowState.MINIMIZED.__doc__ = "Window is minimized."
ApplicationWindowState.NORMAL.__doc__ = "Normal window state."

agcls.AgTypeNameMap["ApplicationWindowState"] = ApplicationWindowState

class WindowArrangeStyle(IntEnum):
    """Window layout styles."""

    CASCADE = 1
    """Child windows are cascaded within the main window."""
    TILED_HORIZONTAL = 2
    """Child windows are tiled horizontally within the main window."""
    TILED_VERTICAL = 3
    """Child windows are tiled vertically within the main window."""

WindowArrangeStyle.CASCADE.__doc__ = "Child windows are cascaded within the main window."
WindowArrangeStyle.TILED_HORIZONTAL.__doc__ = "Child windows are tiled horizontally within the main window."
WindowArrangeStyle.TILED_VERTICAL.__doc__ = "Child windows are tiled vertically within the main window."

agcls.AgTypeNameMap["WindowArrangeStyle"] = WindowArrangeStyle

class WindowDockStyle(IntEnum):
    """Window docking styles."""

    INTEGRATED = 1
    """Child window is integrated into the main window."""
    DOCKED_LEFT = 2
    """Child window is docked to the left side of the within the main window."""
    DOCKED_RIGHT = 3
    """Child window is docked to the right side of the main window."""
    DOCKED_TOP = 4
    """Child window is docked to the top of the main window."""
    DOCKED_BOTTOM = 5
    """Child window is docked to the bottom of the main window."""
    FLOATING = 6
    """Child window is not docked or integrated."""

WindowDockStyle.INTEGRATED.__doc__ = "Child window is integrated into the main window."
WindowDockStyle.DOCKED_LEFT.__doc__ = "Child window is docked to the left side of the within the main window."
WindowDockStyle.DOCKED_RIGHT.__doc__ = "Child window is docked to the right side of the main window."
WindowDockStyle.DOCKED_TOP.__doc__ = "Child window is docked to the top of the main window."
WindowDockStyle.DOCKED_BOTTOM.__doc__ = "Child window is docked to the bottom of the main window."
WindowDockStyle.FLOATING.__doc__ = "Child window is not docked or integrated."

agcls.AgTypeNameMap["WindowDockStyle"] = WindowDockStyle

class WindowArrangeState(IntEnum):
    """Floating state."""

    FLOATED = 1
    """The UI element is floated."""
    DOCKED = 2
    """The UI element is docked."""

WindowArrangeState.FLOATED.__doc__ = "The UI element is floated."
WindowArrangeState.DOCKED.__doc__ = "The UI element is docked."

agcls.AgTypeNameMap["WindowArrangeState"] = WindowArrangeState



class WindowsCollection(SupportsDeleteCallback):
    """Provide methods and properties to manage the application's windows."""

    _num_methods = 7
    _vtable_offset = IDispatch._vtable_offset + IDispatch._num_methods
    _item_method_offset = 1
    _get_count_method_offset = 2
    _arrange_method_offset = 3
    _add_method_offset = 4
    _get__new_enum_method_offset = 5
    _get_item_by_index_method_offset = 6
    _get_item_by_name_method_offset = 7
    _metadata = {
        "iid_data" : (4730401565789584263, 3861368304027982243),
        "vtable_reference" : IDispatch._vtable_offset + IDispatch._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, WindowsCollection)
    def __iter__(self):
        """Create an iterator for the WindowsCollection object."""
        self.__dict__["_enumerator"] = self._new_enum
        self._enumerator.reset()
        return self
    def __next__(self) -> "Window":
        """Return the next element in the collection."""
        if self._enumerator is None:
            raise StopIteration
        nextval = self._enumerator.next()
        if nextval is None:
            raise StopIteration
        return nextval

    _item_metadata = { "offset" : _item_method_offset,
            "arg_types" : (agcom.Variant, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.VariantArg, agmarshall.InterfaceOutArg,) }
    def item(self, index_or_caption:typing.Any) -> "Window":
        """Retrieve a window object."""
        return self._intf.invoke(WindowsCollection._metadata, WindowsCollection._item_metadata, index_or_caption, OutArg())

    _get_count_metadata = { "offset" : _get_count_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def count(self) -> int:
        """Return a total number of window objects in the collection."""
        return self._intf.get_property(WindowsCollection._metadata, WindowsCollection._get_count_metadata)

    _arrange_metadata = { "offset" : _arrange_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.EnumArg(WindowArrangeStyle),) }
    def arrange(self, arrange_style:"WindowArrangeStyle") -> None:
        """Arranges the application windows using the specified style."""
        return self._intf.invoke(WindowsCollection._metadata, WindowsCollection._arrange_metadata, arrange_style)

    _add_metadata = { "offset" : _add_method_offset,
            "arg_types" : (agcom.BSTR, agcom.Variant, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.VariantArg, agmarshall.InterfaceOutArg,) }
    def add(self, plugin_id:str, init_data:typing.Any) -> "Window":
        """Create a new window. The bstrPluginID is a COM ProgID associated with an STK plugin."""
        return self._intf.invoke(WindowsCollection._metadata, WindowsCollection._add_metadata, plugin_id, init_data, OutArg())

    _get__new_enum_metadata = { "offset" : _get__new_enum_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.IEnumVariantArg,) }
    @property
    def _new_enum(self) -> EnumeratorProxy:
        """Enumerates the windows in the collection."""
        return self._intf.get_property(WindowsCollection._metadata, WindowsCollection._get__new_enum_metadata)

    _get_item_by_index_metadata = { "offset" : _get_item_by_index_method_offset,
            "arg_types" : (agcom.INT, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.IntArg, agmarshall.InterfaceOutArg,) }
    def get_item_by_index(self, index:int) -> "Window":
        """Retrieve a window object by index in collection."""
        return self._intf.invoke(WindowsCollection._metadata, WindowsCollection._get_item_by_index_metadata, index, OutArg())

    _get_item_by_name_metadata = { "offset" : _get_item_by_name_method_offset,
            "arg_types" : (agcom.BSTR, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.InterfaceOutArg,) }
    def get_item_by_name(self, name:str) -> "Window":
        """Retrieve a window object by name of window object."""
        return self._intf.invoke(WindowsCollection._metadata, WindowsCollection._get_item_by_name_metadata, name, OutArg())

    __getitem__ = item


    _property_names[count] = "count"
    _property_names[_new_enum] = "_new_enum"

    def __init__(self, source_object=None):
        """Construct an object of type WindowsCollection."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, WindowsCollection)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, WindowsCollection, [WindowsCollection, ])

agcls.AgClassCatalog.add_catalog_entry((5494012632689531786, 14643514705293336469), WindowsCollection)
agcls.AgTypeNameMap["WindowsCollection"] = WindowsCollection

class Window(SupportsDeleteCallback):
    """Represents a window abstraction. Provides methods and properties to manipulate the position and the state of the window."""

    _num_methods = 24
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _get_caption_method_offset = 1
    _set_caption_method_offset = 2
    _activate_method_offset = 3
    _get_window_state_method_offset = 4
    _set_window_state_method_offset = 5
    _close_method_offset = 6
    _get_height_method_offset = 7
    _set_height_method_offset = 8
    _get_width_method_offset = 9
    _set_width_method_offset = 10
    _get_left_method_offset = 11
    _set_left_method_offset = 12
    _get_top_method_offset = 13
    _set_top_method_offset = 14
    _get_dock_style_method_offset = 15
    _set_dock_style_method_offset = 16
    _get_no_workbook_close_method_offset = 17
    _set_no_workbook_close_method_offset = 18
    _get_unpinned_method_offset = 19
    _set_unpinned_method_offset = 20
    _get_can_pin_method_offset = 21
    _get_toolbars_method_offset = 22
    _get_service_by_name_method_offset = 23
    _get_service_by_type_method_offset = 24
    _metadata = {
        "iid_data" : (5238521222474863957, 13235753129659511978),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, Window)

    _get_caption_metadata = { "offset" : _get_caption_method_offset,
            "arg_types" : (POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg,) }
    @property
    def caption(self) -> str:
        """Get or set the window caption. Can only be set within UI plugins for the non unique windows they own."""
        return self._intf.get_property(Window._metadata, Window._get_caption_metadata)

    _set_caption_metadata = { "offset" : _set_caption_method_offset,
            "arg_types" : (agcom.BSTR,),
            "marshallers" : (agmarshall.BStrArg,) }
    @caption.setter
    def caption(self, caption:str) -> None:
        """Get or set  the window caption. Can only be set within UI plugins for the non unique windows they own."""
        return self._intf.set_property(Window._metadata, Window._set_caption_metadata, caption)

    _activate_metadata = { "offset" : _activate_method_offset,
            "arg_types" : (),
            "marshallers" : () }
    def activate(self) -> None:
        """Activates the window."""
        return self._intf.invoke(Window._metadata, Window._activate_metadata, )

    _get_window_state_metadata = { "offset" : _get_window_state_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.EnumArg(ApplicationWindowState),) }
    @property
    def window_state(self) -> "ApplicationWindowState":
        """The window state."""
        return self._intf.get_property(Window._metadata, Window._get_window_state_metadata)

    _set_window_state_metadata = { "offset" : _set_window_state_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.EnumArg(ApplicationWindowState),) }
    @window_state.setter
    def window_state(self, new_value:"ApplicationWindowState") -> None:
        return self._intf.set_property(Window._metadata, Window._set_window_state_metadata, new_value)

    _close_metadata = { "offset" : _close_method_offset,
            "arg_types" : (),
            "marshallers" : () }
    def close(self) -> None:
        """Close the window."""
        return self._intf.invoke(Window._metadata, Window._close_metadata, )

    _get_height_metadata = { "offset" : _get_height_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def height(self) -> int:
        """The window height."""
        return self._intf.get_property(Window._metadata, Window._get_height_metadata)

    _set_height_metadata = { "offset" : _set_height_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @height.setter
    def height(self, new_value:int) -> None:
        return self._intf.set_property(Window._metadata, Window._set_height_metadata, new_value)

    _get_width_metadata = { "offset" : _get_width_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def width(self) -> int:
        """The window width."""
        return self._intf.get_property(Window._metadata, Window._get_width_metadata)

    _set_width_metadata = { "offset" : _set_width_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @width.setter
    def width(self, new_value:int) -> None:
        return self._intf.set_property(Window._metadata, Window._set_width_metadata, new_value)

    _get_left_metadata = { "offset" : _get_left_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def left(self) -> int:
        """The window horizontal position."""
        return self._intf.get_property(Window._metadata, Window._get_left_metadata)

    _set_left_metadata = { "offset" : _set_left_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @left.setter
    def left(self, new_value:int) -> None:
        return self._intf.set_property(Window._metadata, Window._set_left_metadata, new_value)

    _get_top_metadata = { "offset" : _get_top_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def top(self) -> int:
        """The window vertical position."""
        return self._intf.get_property(Window._metadata, Window._get_top_metadata)

    _set_top_metadata = { "offset" : _set_top_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.LongArg,) }
    @top.setter
    def top(self, new_value:int) -> None:
        return self._intf.set_property(Window._metadata, Window._set_top_metadata, new_value)

    _get_dock_style_metadata = { "offset" : _get_dock_style_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.EnumArg(WindowDockStyle),) }
    @property
    def dock_style(self) -> "WindowDockStyle":
        """The window docking style."""
        return self._intf.get_property(Window._metadata, Window._get_dock_style_metadata)

    _set_dock_style_metadata = { "offset" : _set_dock_style_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.EnumArg(WindowDockStyle),) }
    @dock_style.setter
    def dock_style(self, new_value:"WindowDockStyle") -> None:
        return self._intf.set_property(Window._metadata, Window._set_dock_style_metadata, new_value)

    _get_no_workbook_close_metadata = { "offset" : _get_no_workbook_close_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def no_workbook_close(self) -> bool:
        """Whether to close the window when the application workbook is loaded/closed."""
        return self._intf.get_property(Window._metadata, Window._get_no_workbook_close_metadata)

    _set_no_workbook_close_metadata = { "offset" : _set_no_workbook_close_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL,),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @no_workbook_close.setter
    def no_workbook_close(self, new_value:bool) -> None:
        return self._intf.set_property(Window._metadata, Window._set_no_workbook_close_metadata, new_value)

    _get_unpinned_metadata = { "offset" : _get_unpinned_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def unpinned(self) -> bool:
        """The window's pinned state."""
        return self._intf.get_property(Window._metadata, Window._get_unpinned_metadata)

    _set_unpinned_metadata = { "offset" : _set_unpinned_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL,),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @unpinned.setter
    def unpinned(self, new_value:bool) -> None:
        return self._intf.set_property(Window._metadata, Window._set_unpinned_metadata, new_value)

    _get_can_pin_metadata = { "offset" : _get_can_pin_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def can_pin(self) -> bool:
        """Return whether the window supports pinning."""
        return self._intf.get_property(Window._metadata, Window._get_can_pin_metadata)

    _get_toolbars_metadata = { "offset" : _get_toolbars_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.InterfaceOutArg,) }
    @property
    def toolbars(self) -> "ToolbarCollection":
        """Return the window's toolbar collection."""
        return self._intf.get_property(Window._metadata, Window._get_toolbars_metadata)

    _get_service_by_name_metadata = { "offset" : _get_service_by_name_method_offset,
            "arg_types" : (agcom.BSTR, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.InterfaceOutArg,) }
    def get_service_by_name(self, name:str) -> typing.Any:
        """Return a service object that can be accessed at runtime. The method returns null if no service object is associated with the specified symbolic name."""
        return self._intf.invoke(Window._metadata, Window._get_service_by_name_metadata, name, OutArg())

    _get_service_by_type_metadata = { "offset" : _get_service_by_type_method_offset,
            "arg_types" : (agcom.LONG, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.EnumArg(WindowServiceType), agmarshall.InterfaceOutArg,) }
    def get_service_by_type(self, service_type:"WindowServiceType") -> typing.Any:
        """Return a service object that can be accessed at runtime. The method returns null if no service object is associated with the specified service type."""
        return self._intf.invoke(Window._metadata, Window._get_service_by_type_metadata, service_type, OutArg())

    _property_names[caption] = "caption"
    _property_names[window_state] = "window_state"
    _property_names[height] = "height"
    _property_names[width] = "width"
    _property_names[left] = "left"
    _property_names[top] = "top"
    _property_names[dock_style] = "dock_style"
    _property_names[no_workbook_close] = "no_workbook_close"
    _property_names[unpinned] = "unpinned"
    _property_names[can_pin] = "can_pin"
    _property_names[toolbars] = "toolbars"

    def __init__(self, source_object=None):
        """Construct an object of type Window."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, Window)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, Window, [Window, ])

agcls.AgClassCatalog.add_catalog_entry((4826632444527701187, 17778306301041749141), Window)
agcls.AgTypeNameMap["Window"] = Window

class Toolbar(SupportsDeleteCallback):
    """Provide methods and properties to control a toolbar."""

    _num_methods = 6
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _get_identifier_method_offset = 1
    _get_caption_method_offset = 2
    _get_visible_method_offset = 3
    _set_visible_method_offset = 4
    _get_float_state_method_offset = 5
    _set_float_state_method_offset = 6
    _metadata = {
        "iid_data" : (4815534316350549014, 8106580190020797345),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, Toolbar)

    _get_identifier_metadata = { "offset" : _get_identifier_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def identifier(self) -> int:
        """The identity."""
        return self._intf.get_property(Toolbar._metadata, Toolbar._get_identifier_metadata)

    _get_caption_metadata = { "offset" : _get_caption_method_offset,
            "arg_types" : (POINTER(agcom.BSTR),),
            "marshallers" : (agmarshall.BStrArg,) }
    @property
    def caption(self) -> str:
        """The caption."""
        return self._intf.get_property(Toolbar._metadata, Toolbar._get_caption_metadata)

    _get_visible_metadata = { "offset" : _get_visible_method_offset,
            "arg_types" : (POINTER(agcom.VARIANT_BOOL),),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @property
    def visible(self) -> bool:
        """The visibility."""
        return self._intf.get_property(Toolbar._metadata, Toolbar._get_visible_metadata)

    _set_visible_metadata = { "offset" : _set_visible_method_offset,
            "arg_types" : (agcom.VARIANT_BOOL,),
            "marshallers" : (agmarshall.VariantBoolArg,) }
    @visible.setter
    def visible(self, new_value:bool) -> None:
        return self._intf.set_property(Toolbar._metadata, Toolbar._set_visible_metadata, new_value)

    _get_float_state_metadata = { "offset" : _get_float_state_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.EnumArg(WindowArrangeState),) }
    @property
    def float_state(self) -> "WindowArrangeState":
        """The float state."""
        return self._intf.get_property(Toolbar._metadata, Toolbar._get_float_state_metadata)

    _set_float_state_metadata = { "offset" : _set_float_state_method_offset,
            "arg_types" : (agcom.LONG,),
            "marshallers" : (agmarshall.EnumArg(WindowArrangeState),) }
    @float_state.setter
    def float_state(self, new_value:"WindowArrangeState") -> None:
        return self._intf.set_property(Toolbar._metadata, Toolbar._set_float_state_metadata, new_value)

    _property_names[identifier] = "identifier"
    _property_names[caption] = "caption"
    _property_names[visible] = "visible"
    _property_names[float_state] = "float_state"

    def __init__(self, source_object=None):
        """Construct an object of type Toolbar."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, Toolbar)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, Toolbar, [Toolbar, ])

agcls.AgClassCatalog.add_catalog_entry((5472906868102444420, 13479142476234282134), Toolbar)
agcls.AgTypeNameMap["Toolbar"] = Toolbar

class ToolbarCollection(SupportsDeleteCallback):
    """Provide methods and properties to obtain a window's toolbars."""

    _num_methods = 6
    _vtable_offset = IDispatch._vtable_offset + IDispatch._num_methods
    _item_method_offset = 1
    _get_count_method_offset = 2
    _get__new_enum_method_offset = 3
    _get_toolbar_by_id_method_offset = 4
    _get_item_by_index_method_offset = 5
    _get_item_by_name_method_offset = 6
    _metadata = {
        "iid_data" : (5034548498384163675, 2931144109818226324),
        "vtable_reference" : IDispatch._vtable_offset + IDispatch._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, ToolbarCollection)
    def __iter__(self):
        """Create an iterator for the ToolbarCollection object."""
        self.__dict__["_enumerator"] = self._new_enum
        self._enumerator.reset()
        return self
    def __next__(self) -> "Toolbar":
        """Return the next element in the collection."""
        if self._enumerator is None:
            raise StopIteration
        nextval = self._enumerator.next()
        if nextval is None:
            raise StopIteration
        return nextval

    _item_metadata = { "offset" : _item_method_offset,
            "arg_types" : (agcom.Variant, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.VariantArg, agmarshall.InterfaceOutArg,) }
    def item(self, index_or_caption:typing.Any) -> "Toolbar":
        """Retrieve a toolbar object."""
        return self._intf.invoke(ToolbarCollection._metadata, ToolbarCollection._item_metadata, index_or_caption, OutArg())

    _get_count_metadata = { "offset" : _get_count_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def count(self) -> int:
        """Return a total number of toolbars in the collection."""
        return self._intf.get_property(ToolbarCollection._metadata, ToolbarCollection._get_count_metadata)

    _get__new_enum_metadata = { "offset" : _get__new_enum_method_offset,
            "arg_types" : (POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.IEnumVariantArg,) }
    @property
    def _new_enum(self) -> EnumeratorProxy:
        """Enumerates the toolbars in the collection."""
        return self._intf.get_property(ToolbarCollection._metadata, ToolbarCollection._get__new_enum_metadata)

    _get_toolbar_by_id_metadata = { "offset" : _get_toolbar_by_id_method_offset,
            "arg_types" : (agcom.LONG, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.LongArg, agmarshall.InterfaceOutArg,) }
    def get_toolbar_by_id(self, id:int) -> "Toolbar":
        """Return a toolbar object with the specified toolbar identifier. The identifier is a unique number assigned to a toolbar object."""
        return self._intf.invoke(ToolbarCollection._metadata, ToolbarCollection._get_toolbar_by_id_metadata, id, OutArg())

    _get_item_by_index_metadata = { "offset" : _get_item_by_index_method_offset,
            "arg_types" : (agcom.INT, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.IntArg, agmarshall.InterfaceOutArg,) }
    def get_item_by_index(self, index:int) -> "Toolbar":
        """Retrieve a toolbar object based on the index in the collection."""
        return self._intf.invoke(ToolbarCollection._metadata, ToolbarCollection._get_item_by_index_metadata, index, OutArg())

    _get_item_by_name_metadata = { "offset" : _get_item_by_name_method_offset,
            "arg_types" : (agcom.BSTR, POINTER(agcom.PVOID),),
            "marshallers" : (agmarshall.BStrArg, agmarshall.InterfaceOutArg,) }
    def get_item_by_name(self, name:str) -> "Toolbar":
        """Retrieve a toolbar object based on the name of the Toolbar in the collection."""
        return self._intf.invoke(ToolbarCollection._metadata, ToolbarCollection._get_item_by_name_metadata, name, OutArg())

    __getitem__ = item


    _property_names[count] = "count"
    _property_names[_new_enum] = "_new_enum"

    def __init__(self, source_object=None):
        """Construct an object of type ToolbarCollection."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, ToolbarCollection)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, ToolbarCollection, [ToolbarCollection, ])

agcls.AgClassCatalog.add_catalog_entry((5214835483446608103, 6177983444187579524), ToolbarCollection)
agcls.AgTypeNameMap["ToolbarCollection"] = ToolbarCollection

class WindowMapObject(SupportsDeleteCallback):
    """Represents a 2D (Map) window. Provides methods and properties to access the 2D window properties."""

    _num_methods = 1
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _get_map_id_method_offset = 1
    _metadata = {
        "iid_data" : (5665093236705462569, 949456394611833022),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, WindowMapObject)

    _get_map_id_metadata = { "offset" : _get_map_id_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def map_id(self) -> int:
        """A unique identifier associated with the window that can be used with Connect to control the 2D map."""
        return self._intf.get_property(WindowMapObject._metadata, WindowMapObject._get_map_id_metadata)

    _property_names[map_id] = "map_id"

    def __init__(self, source_object=None):
        """Construct an object of type WindowMapObject."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, WindowMapObject)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, WindowMapObject, [WindowMapObject, ])

agcls.AgClassCatalog.add_catalog_entry((5532961742508552268, 7732337666827650452), WindowMapObject)
agcls.AgTypeNameMap["WindowMapObject"] = WindowMapObject

class WindowGlobeObject(SupportsDeleteCallback):
    """Represents a 3D (Globe) window. Provides methods and properties to access the 3D window properties."""

    _num_methods = 1
    _vtable_offset = IUnknown._vtable_offset + IUnknown._num_methods
    _get_scene_id_method_offset = 1
    _metadata = {
        "iid_data" : (5014201186762943933, 15007269609063404450),
        "vtable_reference" : IUnknown._vtable_offset + IUnknown._num_methods - 1,
    }
    _property_names = {}
    def _get_property(self, attrname):
        return get_interface_property(attrname, WindowGlobeObject)

    _get_scene_id_metadata = { "offset" : _get_scene_id_method_offset,
            "arg_types" : (POINTER(agcom.LONG),),
            "marshallers" : (agmarshall.LongArg,) }
    @property
    def scene_id(self) -> int:
        """A unique identifier associated with the window that can be used with Connect to control the 3D globe."""
        return self._intf.get_property(WindowGlobeObject._metadata, WindowGlobeObject._get_scene_id_metadata)

    _property_names[scene_id] = "scene_id"

    def __init__(self, source_object=None):
        """Construct an object of type WindowGlobeObject."""
        SupportsDeleteCallback.__init__(self)
        initialize_from_source_object(self, source_object, WindowGlobeObject)
    def _private_init(self, intf:InterfaceProxy):
        self.__dict__["_intf"] = intf
    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return agcls.compare_com_objects(self, other)
    def __setattr__(self, attrname, value):
        """Attempt to assign an attribute."""
        set_class_attribute(self, attrname, value, WindowGlobeObject, [WindowGlobeObject, ])

agcls.AgClassCatalog.add_catalog_entry((5334286057966533215, 999415816428096669), WindowGlobeObject)
agcls.AgTypeNameMap["WindowGlobeObject"] = WindowGlobeObject


# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from typing import Set, Dict
from functools import wraps

import importlib


def required_package(package_name: str):
    """
    Check if a package is installed by importing it.

    Parameters
    ----------
    package_name: str
        Name of the package to import.

    Raises
    ----------
    ModuleNotFoundError
        Raised when a package is not installed.
    """

    def decorator(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            """Try to import the required package and calls the decorated function."""
            try:
                importlib.import_module(package_name)
                return function(*args, **kwargs)
            except ModuleNotFoundError:
                error_msg = (
                    f"The package \"{package_name}\" is required."
                )
                raise ModuleNotFoundError(error_msg)
        return wrapper

    return decorator


@required_package("numpy")
def to_numpy_array(results: "DataProviderResultDataSetCollection") -> "ndarray":
    import numpy

    results_arr = numpy.array([])

    # create numpy array from row formatted dataset elements
    row_elements = results.to_array()
    unshaped_elements_arr = numpy.array(row_elements)

    # get unique element names and unique element count
    unique_element_names = _get_unique_element_names(results)
    num_unique_columns = len(unique_element_names)

    # reshape to flatten list while preserving the proper column dimensions
    if num_unique_columns:
        results_arr = unshaped_elements_arr.reshape(-1, num_unique_columns)

    return results_arr


@required_package("pandas")
def to_pandas_dataframe(results: "DataProviderResultDataSetCollection", index_element_name: str = None,
                 data_provider_elements: "DataProviderElements" = None) -> "DataFrame":
    import pandas

    results_df = pandas.DataFrame()
    results_arr = to_numpy_array(results)

    if results_arr.size > 0:
        unique_element_names = _get_unique_element_names(results)
        num_unique_elements = len(unique_element_names)

        # Slice element names list to get unique column names in the order that they appear in the DataSet. This
        # ensures that the order of the unique column names is maintained when they are used as columns in the
        # new DataFrame.
        unique_element_names = results.element_names[0:num_unique_elements]

        # normalize element names to mitigate errors working and comparing DataFrame column names as column names are
        # case sensitive
        normalized_unique_element_names = [name.lower() for name in unique_element_names]

        results_df = pandas.DataFrame(data=results_arr, columns=normalized_unique_element_names)

        # set DataFrame index column
        if index_element_name:
            normalized_index_column = None

            # check that element name to be used as the DataFrame index column is valid
            for name in normalized_unique_element_names:
                if index_element_name.lower() == name:
                    normalized_index_column = index_element_name.lower()
                    break

            if normalized_index_column:
                results_df = results_df.set_index(normalized_index_column)
            else:
                element_names_str = ",".join(normalized_unique_element_names)
                error_message = f"\"{index_element_name}\" is not a valid data provider element name. Valid " \
                                f"element names are: {element_names_str}"
                raise ValueError(error_message)

        # map data provider element types to pandas dtypes
        if data_provider_elements:
            dtypes_dict = _map_element_types_to_pandas_dtypes(data_provider_elements,
                                                              index_element_name=index_element_name)

            # Update DataFrame column dtypes with mapped types. If we encounter values that can not be
            # converted such as None or Nan ignore them and return the original object, this allows the caller to
            # determine how to they would like to handle Nan etc values.
            results_df = results_df.astype(dtypes_dict, errors="ignore")

    return results_df


def _get_unique_element_names(results: "DataProviderResultDataSetCollection") -> Set:
    """Return a unique set of element names as a set."""

    unique_element_names = set(results.element_names)

    return unique_element_names


@required_package("numpy")
def _map_element_types_to_pandas_dtypes(data_provider_elements: "DataProviderElements",
                                        index_element_name: str = None) -> Dict[str, object]:
    """
    Return a mapping of STK data provider element names and their types to corresponding pandas dtypes.

    Notes
    -----
    This function requires ``numpy``.
    """
    import numpy
    from ..stkobjects import DataProviderElementType

    dtype_element_name_mapping = dict()

    for element in data_provider_elements:
        element_type = element.type
        normalized_element_name = element.name.lower()
        element_dimensions_name = element.dimension_name.lower()

        # By default to avoid issues with possible leap seconds or other time precision related issues we map date
        # dimension elements as string dtypes in pandas. Future work plans to implement more robust datetime support
        # for pandas.
        if element_type == DataProviderElementType.REAL and element_dimensions_name not in "date":
            pd_dtype = numpy.float64
        elif element_type == DataProviderElementType.INTEGER:
            pd_dtype = numpy.int64
        else:
            # by default make everything else a str, strings like datatime strings can be handled/parsed
            # separately
            pd_dtype = str

        if not index_element_name or normalized_element_name != index_element_name.lower():
            dtype_element_name_mapping[normalized_element_name] = pd_dtype

    return dtype_element_name_mapping



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: AgGrpcServices.proto
# Protobuf Python Version: 5.26.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x41gGrpcServices.proto\x12\x0e\x41gGrpcServices\"+\n\x0bInterfaceID\x12\r\n\x05\x64\x61ta1\x18\x01 \x01(\x06\x12\r\n\x05\x64\x61ta2\x18\x02 \x01(\x06\"E\n\tSTKObject\x12\r\n\x05value\x18\x01 \x01(\x06\x12)\n\x04guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\"\x0e\n\x0c\x45mptyMessage\"\x0b\n\tNullValue\"]\n\x12\x43onnectionMetadata\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x0f\n\x07release\x18\x02 \x01(\r\x12\x0e\n\x06update\x18\x03 \x01(\r\x12\x15\n\rconnection_id\x18\x04 \x01(\r\"a\n\x0fShutDownRequest\x12\x31\n\x0e\x61pp_to_release\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x1b\n\x13shutdown_stkruntime\x18\x02 \x01(\x08\"3\n\tObjReturn\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\"9\n\x0fRefCountRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\"\x1f\n\x0eRefCountReturn\x12\r\n\x05\x63ount\x18\x01 \x01(\r\"C\n\x15\x42\x61tchedReleaseRequest\x12*\n\x07objects\x18\x01 \x03(\x0b\x32\x19.AgGrpcServices.STKObject\"w\n\x18SupportsInterfaceRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\x0einterface_guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\")\n\x17SupportsInterfaceReturn\x12\x0e\n\x06result\x18\x01 \x01(\x08\"\x93\x03\n\x07Variant\x12\x34\n\x06values\x18\x01 \x03(\x0b\x32$.AgGrpcServices.Variant.VariantValue\x12&\n\x1enum_columns_in_repeated_values\x18\x02 \x01(\r\x1a\xa9\x02\n\x0cVariantValue\x12(\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObjectH\x00\x12\x11\n\x07str_val\x18\x02 \x01(\tH\x00\x12\x18\n\x0esigned_int_val\x18\x03 \x01(\x10H\x00\x12\x1a\n\x10unsigned_int_val\x18\x04 \x01(\x06H\x00\x12\x14\n\ndouble_val\x18\x05 \x01(\x01H\x00\x12\x12\n\x08\x62ool_val\x18\x06 \x01(\x08H\x00\x12)\n\x04null\x18\x07 \x01(\x0b\x32\x19.AgGrpcServices.NullValueH\x00\x12/\n\x0cnested_array\x18\x08 \x01(\x0b\x32\x17.AgGrpcServices.VariantH\x00\x12\x17\n\robj_future_id\x18\t \x01(\rH\x00\x42\x07\n\x05Value\"\x80\x01\n\x12GetPropertyRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\x0einterface_guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x03 \x01(\r\"=\n\x11GetPropertyReturn\x12(\n\x07variant\x18\x01 \x01(\x0b\x32\x17.AgGrpcServices.Variant\"\xaa\x01\n\x12SetPropertyRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\x0einterface_guid\x18\x02 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x03 \x01(\r\x12(\n\x07variant\x18\x04 \x01(\x0b\x32\x17.AgGrpcServices.Variant\"\xd8\x01\n\x10STKObjectPromise\x12\n\n\x02id\x18\x01 \x01(\r\x12(\n\x03obj\x18\x02 \x01(\x0b\x32\x19.AgGrpcServices.STKObjectH\x00\x12\x17\n\robj_future_id\x18\x03 \x01(\rH\x00\x12\x33\n\x0einterface_guid\x18\x04 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x05 \x01(\r\x12%\n\x04\x61rgs\x18\x06 \x03(\x0b\x32\x17.AgGrpcServices.VariantB\n\n\x08instance\"\xe4\x01\n\rInvokeRequest\x12(\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObjectH\x00\x12\x32\n\x06\x66uture\x18\x02 \x01(\x0b\x32 .AgGrpcServices.STKObjectPromiseH\x00\x12\x33\n\x0einterface_guid\x18\x03 \x01(\x0b\x32\x1b.AgGrpcServices.InterfaceID\x12\r\n\x05index\x18\x04 \x01(\r\x12%\n\x04\x61rgs\x18\x05 \x03(\x0b\x32\x17.AgGrpcServices.VariantB\n\n\x08instance\"<\n\x0cInvokeReturn\x12,\n\x0breturn_vals\x18\x01 \x03(\x0b\x32\x17.AgGrpcServices.Variant\"e\n\x14\x42\x61tchedInvokeRequest\x12/\n\x08requests\x18\x01 \x03(\x0b\x32\x1d.AgGrpcServices.InvokeRequest\x12\x1c\n\x14\x66uture_ids_to_return\x18\x02 \x03(\r\"\xab\x01\n\x13\x42\x61tchedInvokeReturn\x12\x46\n\rbound_futures\x18\x01 \x03(\x0b\x32/.AgGrpcServices.BatchedInvokeReturn.BoundFuture\x1aL\n\x0b\x42oundFuture\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x15\n\robj_future_id\x18\x02 \x01(\r\"N\n\x11\x43ollectionRequest\x12&\n\x03obj\x18\x01 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x11\n\tmax_items\x18\x02 \x01(\r\"S\n\x10\x43ollectionReturn\x12&\n\x05items\x18\x01 \x03(\x0b\x32\x17.AgGrpcServices.Variant\x12\x17\n\x0f\x64one_collecting\x18\x02 \x01(\x08\"\x9a\x01\n\x10SubscriptionData\x12\x15\n\revent_loop_id\x18\x01 \x01(\r\x12&\n\x03obj\x18\x02 \x01(\x0b\x32\x19.AgGrpcServices.STKObject\x12\x33\n\revent_handler\x18\x03 \x01(\x0e\x32\x1c.AgGrpcServices.EventHandler\x12\x12\n\nevent_name\x18\x04 \x01(\t\"8\n\rEventLoopData\x12\x15\n\revent_loop_id\x18\x01 \x01(\r\x12\x10\n\x08\x65vent_id\x18\x02 \x01(\r\"\x8d\x01\n\x11\x45ventLoopResponse\x12\x36\n\x0csubscription\x18\x01 \x01(\x0b\x32 .AgGrpcServices.SubscriptionData\x12.\n\rcallback_args\x18\x02 \x03(\x0b\x32\x17.AgGrpcServices.Variant\x12\x10\n\x08\x65vent_id\x18\x03 \x01(\r*\x83\x02\n\x0c\x45ventHandler\x12\x1b\n\x17\x65IAgStkObjectRootEvents\x10\x00\x12\x1d\n\x19\x65IAgSTKXApplicationEvents\x10\x01\x12\x1e\n\x1a\x65IAgStkGraphicsSceneEvents\x10\x02\x12$\n eIAgStkGraphicsKmlGraphicsEvents\x10\x03\x12(\n$eIAgStkGraphicsImageCollectionEvents\x10\x04\x12*\n&eIAgStkGraphicsTerrainCollectionEvents\x10\x05\x12\x1b\n\x17\x65IAgUiApplicationEvents\x10\x06\x32\xd7\x0c\n\x0eSTKGrpcService\x12[\n\x15GetConnectionMetadata\x12\x1c.AgGrpcServices.EmptyMessage\x1a\".AgGrpcServices.ConnectionMetadata\"\x00\x12Q\n\x0eShutDownServer\x12\x1f.AgGrpcServices.ShutDownRequest\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12N\n\x11GetStkApplication\x12\x1c.AgGrpcServices.EmptyMessage\x1a\x19.AgGrpcServices.ObjReturn\"\x00\x12J\n\rEngineNewRoot\x12\x1c.AgGrpcServices.EmptyMessage\x1a\x19.AgGrpcServices.ObjReturn\"\x00\x12Q\n\x14\x45ngineNewRootContext\x12\x1c.AgGrpcServices.EmptyMessage\x1a\x19.AgGrpcServices.ObjReturn\"\x00\x12K\n\x06\x41\x64\x64Ref\x12\x1f.AgGrpcServices.RefCountRequest\x1a\x1e.AgGrpcServices.RefCountReturn\"\x00\x12L\n\x07Release\x12\x1f.AgGrpcServices.RefCountRequest\x1a\x1e.AgGrpcServices.RefCountReturn\"\x00\x12W\n\x0e\x42\x61tchedRelease\x12%.AgGrpcServices.BatchedReleaseRequest\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12h\n\x11SupportsInterface\x12(.AgGrpcServices.SupportsInterfaceRequest\x1a\'.AgGrpcServices.SupportsInterfaceReturn\"\x00\x12G\n\x06Invoke\x12\x1d.AgGrpcServices.InvokeRequest\x1a\x1c.AgGrpcServices.InvokeReturn\"\x00\x12V\n\x0bGetProperty\x12\".AgGrpcServices.GetPropertyRequest\x1a!.AgGrpcServices.GetPropertyReturn\"\x00\x12Q\n\x0bSetProperty\x12\".AgGrpcServices.SetPropertyRequest\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12\\\n\x13\x45numerateCollection\x12!.AgGrpcServices.CollectionRequest\x1a .AgGrpcServices.CollectionReturn\"\x00\x12\\\n\rBatchedInvoke\x12$.AgGrpcServices.BatchedInvokeRequest\x1a#.AgGrpcServices.BatchedInvokeReturn\"\x00\x12U\n\x0eStartEventLoop\x12\x1c.AgGrpcServices.EmptyMessage\x1a!.AgGrpcServices.EventLoopResponse\"\x00\x30\x01\x12N\n\rStopEventLoop\x12\x1d.AgGrpcServices.EventLoopData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12Q\n\x10\x41\x63knowledgeEvent\x12\x1d.AgGrpcServices.EventLoopData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12M\n\tSubscribe\x12 .AgGrpcServices.SubscriptionData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x12O\n\x0bUnsubscribe\x12 .AgGrpcServices.SubscriptionData\x1a\x1c.AgGrpcServices.EmptyMessage\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'AgGrpcServices_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EVENTHANDLER']._serialized_start=2844
  _globals['_EVENTHANDLER']._serialized_end=3103
  _globals['_INTERFACEID']._serialized_start=40
  _globals['_INTERFACEID']._serialized_end=83
  _globals['_STKOBJECT']._serialized_start=85
  _globals['_STKOBJECT']._serialized_end=154
  _globals['_EMPTYMESSAGE']._serialized_start=156
  _globals['_EMPTYMESSAGE']._serialized_end=170
  _globals['_NULLVALUE']._serialized_start=172
  _globals['_NULLVALUE']._serialized_end=183
  _globals['_CONNECTIONMETADATA']._serialized_start=185
  _globals['_CONNECTIONMETADATA']._serialized_end=278
  _globals['_SHUTDOWNREQUEST']._serialized_start=280
  _globals['_SHUTDOWNREQUEST']._serialized_end=377
  _globals['_OBJRETURN']._serialized_start=379
  _globals['_OBJRETURN']._serialized_end=430
  _globals['_REFCOUNTREQUEST']._serialized_start=432
  _globals['_REFCOUNTREQUEST']._serialized_end=489
  _globals['_REFCOUNTRETURN']._serialized_start=491
  _globals['_REFCOUNTRETURN']._serialized_end=522
  _globals['_BATCHEDRELEASEREQUEST']._serialized_start=524
  _globals['_BATCHEDRELEASEREQUEST']._serialized_end=591
  _globals['_SUPPORTSINTERFACEREQUEST']._serialized_start=593
  _globals['_SUPPORTSINTERFACEREQUEST']._serialized_end=712
  _globals['_SUPPORTSINTERFACERETURN']._serialized_start=714
  _globals['_SUPPORTSINTERFACERETURN']._serialized_end=755
  _globals['_VARIANT']._serialized_start=758
  _globals['_VARIANT']._serialized_end=1161
  _globals['_VARIANT_VARIANTVALUE']._serialized_start=864
  _globals['_VARIANT_VARIANTVALUE']._serialized_end=1161
  _globals['_GETPROPERTYREQUEST']._serialized_start=1164
  _globals['_GETPROPERTYREQUEST']._serialized_end=1292
  _globals['_GETPROPERTYRETURN']._serialized_start=1294
  _globals['_GETPROPERTYRETURN']._serialized_end=1355
  _globals['_SETPROPERTYREQUEST']._serialized_start=1358
  _globals['_SETPROPERTYREQUEST']._serialized_end=1528
  _globals['_STKOBJECTPROMISE']._serialized_start=1531
  _globals['_STKOBJECTPROMISE']._serialized_end=1747
  _globals['_INVOKEREQUEST']._serialized_start=1750
  _globals['_INVOKEREQUEST']._serialized_end=1978
  _globals['_INVOKERETURN']._serialized_start=1980
  _globals['_INVOKERETURN']._serialized_end=2040
  _globals['_BATCHEDINVOKEREQUEST']._serialized_start=2042
  _globals['_BATCHEDINVOKEREQUEST']._serialized_end=2143
  _globals['_BATCHEDINVOKERETURN']._serialized_start=2146
  _globals['_BATCHEDINVOKERETURN']._serialized_end=2317
  _globals['_BATCHEDINVOKERETURN_BOUNDFUTURE']._serialized_start=2241
  _globals['_BATCHEDINVOKERETURN_BOUNDFUTURE']._serialized_end=2317
  _globals['_COLLECTIONREQUEST']._serialized_start=2319
  _globals['_COLLECTIONREQUEST']._serialized_end=2397
  _globals['_COLLECTIONRETURN']._serialized_start=2399
  _globals['_COLLECTIONRETURN']._serialized_end=2482
  _globals['_SUBSCRIPTIONDATA']._serialized_start=2485
  _globals['_SUBSCRIPTIONDATA']._serialized_end=2639
  _globals['_EVENTLOOPDATA']._serialized_start=2641
  _globals['_EVENTLOOPDATA']._serialized_end=2697
  _globals['_EVENTLOOPRESPONSE']._serialized_start=2700
  _globals['_EVENTLOOPRESPONSE']._serialized_end=2841
  _globals['_STKGRPCSERVICE']._serialized_start=3106
  _globals['_STKGRPCSERVICE']._serialized_end=4729
# @@protoc_insertion_point(module_scope)



# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from __future__ import annotations

__all__ = [ "ISTKObjectRootEventHandler",
            "ISTKXApplicationEventHandler",
            "IUiAxGraphics2DCntrlEventHandler",
            "IUiAxGraphics3DCntrlEventHandler",
            "ISceneEventHandler",
            "IKmlGraphicsEventHandler",
            "IImageCollectionEventHandler",
            "ITerrainOverlayCollectionEventHandler"]

import typing

from .comutil                import IUnknown
from .comevents import (COMEventHandlerImpl, ISTKXApplicationEventCOMHandler,
                        IImageCollectionEventCOMHandler,
                        IKmlGraphicsEventCOMHandler,
                        ISceneEventCOMHandler,
                        ITerrainOverlayCollectionEventCOMHandler,
                        ISTKObjectRootEventCOMHandler, IGraphics2DControlEventCOMHandler,
                        IGraphics3DControlEventCOMHandler)
from ..utilities.exceptions import STKAttributeError, STKEventsAPIError, STKRuntimeError

try:
    from .grpcutil   import GrpcInterface
    from .grpcevents import (GrpcEventHandlerImpl, ISTKXApplicationEventGrpcHandler,
                         IImageCollectionEventGrpcHandler,
                         IKmlGraphicsEventGrpcHandler, ISceneEventGrpcHandler,
                         ITerrainOverlayCollectionEventGrpcHandler,
                         ISTKObjectRootEventGrpcHandler)
except:
    class GrpcInterface(object):
        def __init__(self):
            pass

invalid_use_exception = STKEventsAPIError("Use operator += to register an event callback or operator -= to unregister the callback.")

class _EventSubscriptionManagerImpl(object):
    def __init__(self):
        self._next_id = 0
        self._handlers: typing.Dict[int, STKEventSubscriber] = {}

    def subscribe(self, handler: STKEventSubscriber) -> int:
        self._next_id = self._next_id + 1
        self._handlers[self._next_id] = handler
        handler._subscribe_impl()
        return self._next_id

    def unsubscribe(self, id:int):
        if id in self._handlers:
            self._handlers[id]._unsubscribe_impl()
            del(self._handlers[id])

    def unsubscribe_all(self):
        ids = [id for id in self._handlers]
        for id in ids:
            self.unsubscribe(id)

EventSubscriptionManager = _EventSubscriptionManagerImpl()

class STKEventSubscriber(object):
    def __init__(self, impl: COMEventHandlerImpl | GrpcEventHandlerImpl):
        self.__dict__["_event_manager_id"] = None
        self.__dict__["_impl"]: COMEventHandlerImpl | GrpcEventHandlerImpl = impl
        self.subscribe()

    def __del__(self):
        self.unsubscribe()
        del(self._impl)

    def subscribe(self):
        """Use to re-subscribe to events after calling Unsubscribe.  This class is initialized as subscribed when returned from STKObjectRoot.Subscribe()."""
        if self._event_manager_id is None:
            self.__dict__["_event_manager_id"] = EventSubscriptionManager.subscribe(self)

    def _subscribe_impl(self):
        """Private method, called by EventSubscriptionManager"""
        impl : COMEventHandlerImpl | GrpcEventHandlerImpl = self._impl
        impl.subscribe()

    def unsubscribe(self):
        """Unsubscribe from events."""
        if self._event_manager_id is not None:
            EventSubscriptionManager.unsubscribe(self._event_manager_id)
            self.__dict__["_event_manager_id"] = None

    def _unsubscribe_impl(self):
        """Private method, called by EventSubscriptionManager"""
        impl : COMEventHandlerImpl | GrpcEventHandlerImpl = self._impl
        impl.unsubscribe()

class _STKEvent(object):
    def __init__(self):
        self.__dict__["_callbacks"] = list()
        self.__dict__["_iadd_callback"] = None
        self.__dict__["_isub_callback"] = None

    def __eq__(self, other):
        raise invalid_use_exception

    def __setattr__(self, attrname, value):
        raise invalid_use_exception

    def __iadd__(self, callback):
        if callback not in self._callbacks:
            self._callbacks.append(callback)
            if self._iadd_callback is not None:
                self._iadd_callback(len(self._callbacks))
        return self

    def __isub__(self, callback):
        if callback in self._callbacks:
            self._callbacks.remove(callback)
            if self._isub_callback is not None:
                self._isub_callback(len(self._callbacks))
        return self

    def _set_iadd_callback(self, iadd_callback):
        self.__dict__["_iadd_callback"] = iadd_callback

    def _set_isub_callback(self, isub_callback):
        self.__dict__["_isub_callback"] = isub_callback

    def _safe_assign(self, rhs):
        if type(rhs)==_STKEvent:
            self.__dict__["_callbacks"] = rhs._callbacks.copy()
        else:
            raise invalid_use_exception

################################################################################
#          IAgStkObjectRootEvents
################################################################################

class ISTKObjectRootEventHandler(STKEventSubscriber):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["OnScenarioNew"]              = _STKEvent()
        self._events["OnScenarioLoad"]             = _STKEvent()
        self._events["OnScenarioClose"]            = _STKEvent()
        self._events["OnScenarioSave"]             = _STKEvent()
        self._events["OnLogMessage"]               = _STKEvent()
        self._events["OnAnimUpdate"]               = _STKEvent()
        self._events["OnStkObjectAdded"]           = _STKEvent()
        self._events["OnStkObjectDeleted"]         = _STKEvent()
        self._events["OnStkObjectRenamed"]         = _STKEvent()
        self._events["OnAnimationPlayback"]        = _STKEvent()
        self._events["OnAnimationRewind"]          = _STKEvent()
        self._events["OnAnimationPause"]           = _STKEvent()
        self._events["OnScenarioBeforeSave"]       = _STKEvent()
        self._events["OnAnimationStep"]            = _STKEvent()
        self._events["OnAnimationStepBack"]        = _STKEvent()
        self._events["OnAnimationSlower"]          = _STKEvent()
        self._events["OnAnimationFaster"]          = _STKEvent()
        self._events["OnPercentCompleteUpdate"]    = _STKEvent()
        self._events["OnPercentCompleteEnd"]       = _STKEvent()
        self._events["OnPercentCompleteBegin"]     = _STKEvent()
        self._events["OnStkObjectChanged"]         = _STKEvent()
        self._events["OnScenarioBeforeClose"]      = _STKEvent()
        self._events["OnStkObjectPreDelete"]       = _STKEvent()
        self._events["OnStkObjectStart3dEditing"]  = _STKEvent()
        self._events["OnStkObjectStop3dEditing"]   = _STKEvent()
        self._events["OnStkObjectApply3dEditing"]  = _STKEvent()
        self._events["OnStkObjectCancel3dEditing"] = _STKEvent()
        self._events["OnStkObjectPreCut"]          = _STKEvent()
        self._events["OnStkObjectCopy"]            = _STKEvent()
        self._events["OnStkObjectPaste"]           = _STKEvent()
        if type(interface)==IUnknown:
            impl = ISTKObjectRootEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            impl = ISTKObjectRootEventGrpcHandler(interface, self._events)
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create ISTKObjectRootEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        if attrname in ISTKObjectRootEventHandler.__dict__ and type(ISTKObjectRootEventHandler.__dict__[attrname]) == property:
            ISTKObjectRootEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in STKObjectRootEvents.")

    @property
    def on_scenario_new(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioNew(Path:str) -> None]"""
        return self._events["OnScenarioNew"]

    @on_scenario_new.setter
    def on_scenario_new(self, callback):
        self._events["OnScenarioNew"]._safe_assign(callback)

    @property
    def on_scenario_load(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioLoad(Path:str) -> None]"""
        return self._events["OnScenarioLoad"]

    @on_scenario_load.setter
    def on_scenario_load(self, callback):
        self._events["OnScenarioLoad"]._safe_assign(callback)

    @property
    def on_scenario_close(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioClose() -> None]"""
        return self._events["OnScenarioClose"]

    @on_scenario_close.setter
    def on_scenario_close(self, callback):
        self._events["OnScenarioClose"]._safe_assign(callback)

    @property
    def on_scenario_save(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioSave(Path:str) -> None]"""
        return self._events["OnScenarioSave"]

    @on_scenario_save.setter
    def on_scenario_save(self, callback):
        self._events["OnScenarioSave"]._safe_assign(callback)

    @property
    def on_log_message(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnLogMessage(message:str, msgType:"LogMessageType", errorCode:int, fileName:str, lineNo:int, dispID:"LogMessageDisplayID") -> None]"""
        return self._events["OnLogMessage"]

    @on_log_message.setter
    def on_log_message(self, callback):
        self._events["OnLogMessage"]._safe_assign(callback)

    @property
    def on_anim_update(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimUpdate(timeEpSec:float) -> None]"""
        return self._events["OnAnimUpdate"]

    @on_anim_update.setter
    def on_anim_update(self, callback):
        self._events["OnAnimUpdate"]._safe_assign(callback)

    @property
    def on_stk_object_added(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectAdded(Sender:typing.Any) -> None]"""
        return self._events["OnStkObjectAdded"]

    @on_stk_object_added.setter
    def on_stk_object_added(self, callback):
        self._events["OnStkObjectAdded"]._safe_assign(callback)

    @property
    def on_stk_object_deleted(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectDeleted(Sender:typing.Any) -> None]"""
        return self._events["OnStkObjectDeleted"]

    @on_stk_object_deleted.setter
    def on_stk_object_deleted(self, callback):
        self._events["OnStkObjectDeleted"]._safe_assign(callback)

    @property
    def on_stk_object_renamed(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectRenamed(Sender:typing.Any, OldPath:str, NewPath:str) -> None]"""
        return self._events["OnStkObjectRenamed"]

    @on_stk_object_renamed.setter
    def on_stk_object_renamed(self, callback):
        self._events["OnStkObjectRenamed"]._safe_assign(callback)

    @property
    def on_animation_playback(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationPlayback(CurrentTime:float, eAction:"AnimationActionType", eDirection:"AnimationDirectionType") -> None]"""
        return self._events["OnAnimationPlayback"]

    @on_animation_playback.setter
    def on_animation_playback(self, callback):
        self._events["OnAnimationPlayback"]._safe_assign(callback)

    @property
    def on_animation_rewind(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationRewind() -> None]"""
        return self._events["OnAnimationRewind"]

    @on_animation_rewind.setter
    def on_animation_rewind(self, callback):
        self._events["OnAnimationRewind"]._safe_assign(callback)

    @property
    def on_animation_pause(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationPause(CurrentTime:float) -> None]"""
        return self._events["OnAnimationPause"]

    @on_animation_pause.setter
    def on_animation_pause(self, callback):
        self._events["OnAnimationPause"]._safe_assign(callback)

    @property
    def on_scenario_before_save(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioBeforeSave(pArgs:"ScenarioBeforeSaveEventArguments") -> None]"""
        return self._events["OnScenarioBeforeSave"]

    @on_scenario_before_save.setter
    def on_scenario_before_save(self, callback):
        self._events["OnScenarioBeforeSave"]._safe_assign(callback)

    @property
    def on_animation_step(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationStep(CurrentTime:float) -> None]"""
        return self._events["OnAnimationStep"]

    @on_animation_step.setter
    def on_animation_step(self, callback):
        self._events["OnAnimationStep"]._safe_assign(callback)

    @property
    def on_animation_step_back(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationStepBack(CurrentTime:float) -> None]"""
        return self._events["OnAnimationStepBack"]

    @on_animation_step_back.setter
    def on_animation_step_back(self, callback):
        self._events["OnAnimationStepBack"]._safe_assign(callback)

    @property
    def on_animation_slower(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationSlower() -> None]"""
        return self._events["OnAnimationSlower"]

    @on_animation_slower.setter
    def on_animation_slower(self, callback):
        self._events["OnAnimationSlower"]._safe_assign(callback)

    @property
    def on_animation_faster(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimationFaster() -> None]"""
        return self._events["OnAnimationFaster"]

    @on_animation_faster.setter
    def on_animation_faster(self, callback):
        self._events["OnAnimationFaster"]._safe_assign(callback)

    @property
    def on_percent_complete_update(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnPercentCompleteUpdate(pArgs:"ProgressBarEventArguments") -> None]"""
        return self._events["OnPercentCompleteUpdate"]

    @on_percent_complete_update.setter
    def on_percent_complete_update(self, callback):
        self._events["OnPercentCompleteUpdate"]._safe_assign(callback)

    @property
    def on_percent_complete_end(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnPercentCompleteEnd() -> None]"""
        return self._events["OnPercentCompleteEnd"]

    @on_percent_complete_end.setter
    def on_percent_complete_end(self, callback):
        self._events["OnPercentCompleteEnd"]._safe_assign(callback)

    @property
    def on_percent_complete_begin(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnPercentCompleteBegin() -> None]"""
        return self._events["OnPercentCompleteBegin"]

    @on_percent_complete_begin.setter
    def on_percent_complete_begin(self, callback):
        self._events["OnPercentCompleteBegin"]._safe_assign(callback)

    @property
    def on_stk_object_changed(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectChanged(pArgs:"STKObjectChangedEventArguments") -> None]"""
        return self._events["OnStkObjectChanged"]

    @on_stk_object_changed.setter
    def on_stk_object_changed(self, callback):
        self._events["OnStkObjectChanged"]._safe_assign(callback)

    @property
    def on_scenario_before_close(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioBeforeClose() -> None]"""
        return self._events["OnScenarioBeforeClose"]

    @on_scenario_before_close.setter
    def on_scenario_before_close(self, callback):
        self._events["OnScenarioBeforeClose"]._safe_assign(callback)

    @property
    def on_stk_object_pre_delete(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectPreDelete(pArgs:"STKObjectPreDeleteEventArguments") -> None]"""
        return self._events["OnStkObjectPreDelete"]

    @on_stk_object_pre_delete.setter
    def on_stk_object_pre_delete(self, callback):
        self._events["OnStkObjectPreDelete"]._safe_assign(callback)

    @property
    def on_stk_object_start_3d_editing(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectStart3dEditing(Path:str) -> None]"""
        return self._events["OnStkObjectStart3dEditing"]

    @on_stk_object_start_3d_editing.setter
    def on_stk_object_start_3d_editing(self, callback):
        self._events["OnStkObjectStart3dEditing"]._safe_assign(callback)

    @property
    def on_stk_object_stop_3d_editing(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectStop3dEditing(Path:str) -> None]"""
        return self._events["OnStkObjectStop3dEditing"]

    @on_stk_object_stop_3d_editing.setter
    def on_stk_object_stop_3d_editing(self, callback):
        self._events["OnStkObjectStop3dEditing"]._safe_assign(callback)

    @property
    def on_stk_object_apply_3d_editing(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectApply3dEditing(Path:str) -> None]"""
        return self._events["OnStkObjectApply3dEditing"]

    @on_stk_object_apply_3d_editing.setter
    def on_stk_object_apply_3d_editing(self, callback):
        self._events["OnStkObjectApply3dEditing"]._safe_assign(callback)

    @property
    def on_stk_object_cancel_3d_editing(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectCancel3dEditing(Path:str) -> None]"""
        return self._events["OnStkObjectCancel3dEditing"]

    @on_stk_object_cancel_3d_editing.setter
    def on_stk_object_cancel_3d_editing(self, callback):
        self._events["OnStkObjectCancel3dEditing"]._safe_assign(callback)

    @property
    def on_stk_object_pre_cut(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectPreCut(pArgs:"STKObjectCutCopyPasteEventArguments") -> None]"""
        return self._events["OnStkObjectPreCut"]

    @on_stk_object_pre_cut.setter
    def on_stk_object_pre_cut(self, callback):
        self._events["OnStkObjectPreCut"]._safe_assign(callback)

    @property
    def on_stk_object_copy(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectCopy(pArgs:"STKObjectCutCopyPasteEventArguments") -> None]"""
        return self._events["OnStkObjectCopy"]

    @on_stk_object_copy.setter
    def on_stk_object_copy(self, callback):
        self._events["OnStkObjectCopy"]._safe_assign(callback)

    @property
    def on_stk_object_paste(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnStkObjectPaste(pArgs:"STKObjectCutCopyPasteEventArguments") -> None]"""
        return self._events["OnStkObjectPaste"]

    @on_stk_object_paste.setter
    def on_stk_object_paste(self, callback):
        self._events["OnStkObjectPaste"]._safe_assign(callback)


################################################################################
#          IAgSTKXApplicationEvents
################################################################################

class ISTKXApplicationEventHandler(STKEventSubscriber):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["OnScenarioNew"]                = _STKEvent()
        self._events["OnScenarioLoad"]               = _STKEvent()
        self._events["OnScenarioClose"]              = _STKEvent()
        self._events["OnScenarioSave"]               = _STKEvent()
        self._events["OnLogMessage"]                 = _STKEvent()
        self._events["OnAnimUpdate"]                 = _STKEvent()
        self._events["OnNewGlobeCtrlRequest"]        = _STKEvent()
        self._events["OnNewMapCtrlRequest"]          = _STKEvent()
        self._events["OnBeforeNewScenario"]          = _STKEvent()
        self._events["OnBeforeLoadScenario"]         = _STKEvent()
        self._events["OnBeginScenarioClose"]         = _STKEvent()
        self._events["OnNewGfxAnalysisCtrlRequest"]  = _STKEvent()
        self._events["OnSSLCertificateServerError"]  = _STKEvent()
        self._events["OnConControlQuitReceived"]     = _STKEvent()
        if type(interface)==IUnknown:
            impl = ISTKXApplicationEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            impl = ISTKXApplicationEventGrpcHandler(interface, self._events)
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create ISTKXApplicationEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        if attrname in ISTKXApplicationEventHandler.__dict__ and type(ISTKXApplicationEventHandler.__dict__[attrname]) == property:
            ISTKXApplicationEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in STKXApplicationEvents.")

    @property
    def on_scenario_new(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioNew(Path:str) -> None]"""
        return self._events["OnScenarioNew"]

    @on_scenario_new.setter
    def on_scenario_new(self, callback):
        self._events["OnScenarioNew"]._safe_assign(callback)

    @property
    def on_scenario_load(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioLoad(Path:str) -> None]"""
        return self._events["OnScenarioLoad"]

    @on_scenario_load.setter
    def on_scenario_load(self, callback):
        self._events["OnScenarioLoad"]._safe_assign(callback)

    @property
    def on_scenario_close(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioClose() -> None]"""
        return self._events["OnScenarioClose"]

    @on_scenario_close.setter
    def on_scenario_close(self, callback):
        self._events["OnScenarioClose"]._safe_assign(callback)

    @property
    def OnScenarioSave(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnScenarioSave(Path:str) -> None]"""
        return self._events["OnScenarioSave"]

    @OnScenarioSave.setter
    def OnScenarioSave(self, callback):
        self._events["OnScenarioSave"]._safe_assign(callback)

    @property
    def on_log_message(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnLogMessage(message:str, msgType:"LogMessageType", errorCode:int, fileName:str, lineNo:int, dispID:"LogMessageDisplayID") -> None]"""
        return self._events["OnLogMessage"]

    @on_log_message.setter
    def on_log_message(self, callback):
        self._events["OnLogMessage"]._safe_assign(callback)

    @property
    def on_anim_update(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnAnimUpdate(timeEpSec:float) -> None]"""
        return self._events["OnAnimUpdate"]

    @on_anim_update.setter
    def on_anim_update(self, callback):
        self._events["OnAnimUpdate"]._safe_assign(callback)

    @property
    def on_new_globe_ctrl_request(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnNewGlobeCtrlRequest(SceneID:int) -> None]"""
        return self._events["OnNewGlobeCtrlRequest"]

    @on_new_globe_ctrl_request.setter
    def on_new_globe_ctrl_request(self, callback):
        self._events["OnNewGlobeCtrlRequest"]._safe_assign(callback)

    @property
    def on_new_map_ctrl_request(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnNewMapCtrlRequest(WinID:int) -> None]"""
        return self._events["OnNewMapCtrlRequest"]

    @on_new_map_ctrl_request.setter
    def on_new_map_ctrl_request(self, callback):
        self._events["OnNewMapCtrlRequest"]._safe_assign(callback)

    @property
    def on_before_new_scenario(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnBeforeNewScenario(Scenario:str) -> None]"""
        return self._events["OnBeforeNewScenario"]

    @on_before_new_scenario.setter
    def on_before_new_scenario(self, callback):
        self._events["OnBeforeNewScenario"]._safe_assign(callback)

    @property
    def on_before_load_scenario(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnBeforeLoadScenario(Scenario:str) -> None]"""
        return self._events["OnBeforeLoadScenario"]

    @on_before_load_scenario.setter
    def on_before_load_scenario(self, callback):
        self._events["OnBeforeLoadScenario"]._safe_assign(callback)

    @property
    def on_begin_scenario_close(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnBeginScenarioClose() -> None]"""
        return self._events["OnBeginScenarioClose"]

    @on_begin_scenario_close.setter
    def on_begin_scenario_close(self, callback):
        self._events["OnBeginScenarioClose"]._safe_assign(callback)

    @property
    def on_new_gfx_analysis_ctrl_request(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnNewGfxAnalysisCtrlRequest(SceneID:int, GfxAnalysisMode:"Graphics2DAnalysisMode") -> None]"""
        return self._events["OnNewGfxAnalysisCtrlRequest"]

    @on_new_gfx_analysis_ctrl_request.setter
    def on_new_gfx_analysis_ctrl_request(self, callback):
        self._events["OnNewGfxAnalysisCtrlRequest"]._safe_assign(callback)

    @property
    def on_ssl_certificate_server_error(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnSSLCertificateServerError(pArgs:"STKXSSLCertificateErrorEventArgs") -> None]"""
        return self._events["OnSSLCertificateServerError"]

    @on_ssl_certificate_server_error.setter
    def on_ssl_certificate_server_error(self, callback):
        self._events["OnSSLCertificateServerError"]._safe_assign(callback)

    @property
    def on_con_control_quit_received(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnConControlQuitReceived(pArgs:"STKXConControlQuitReceivedEventArgs") -> None]"""
        return self._events["OnConControlQuitReceived"]

    @on_con_control_quit_received.setter
    def on_con_control_quit_received(self, callback):
        self._events["OnConControlQuitReceived"]._safe_assign(callback)


################################################################################
#          ActiveX controls
################################################################################

class IUiAxStockEventHandler(object):
    def __init__(self):
        self._events["KeyDown"]      = _STKEvent()
        self._events["KeyPress"]     = _STKEvent()
        self._events["KeyUp"]        = _STKEvent()
        self._events["Click"]        = _STKEvent()
        self._events["DblClick"]     = _STKEvent()
        self._events["MouseDown"]    = _STKEvent()
        self._events["MouseMove"]    = _STKEvent()
        self._events["MouseUp"]      = _STKEvent()
        self._events["OLEDragDrop"]  = _STKEvent()
        self._events["MouseWheel"]   = _STKEvent()

    def __setattr__(self, attrname, value):
        if attrname in IUiAxStockEventHandler.__dict__ and type(IUiAxStockEventHandler.__dict__[attrname]) == property:
            IUiAxStockEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in IAgUiAxStockEvents.")

    @property
    def key_down(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [KeyDown(KeyCode:int, Shift:int) -> None]"""
        return self._events["KeyDown"]

    @key_down.setter
    def key_down(self, callback):
        self._events["KeyDown"]._safe_assign(callback)

    @property
    def key_press(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [KeyPress(KeyAscii:int) -> None]"""
        return self._events["KeyPress"]

    @key_press.setter
    def key_press(self, callback):
        self._events["KeyPress"]._safe_assign(callback)

    @property
    def key_up(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [KeyUp(KeyCode:int, Shift:int) -> None]"""
        return self._events["KeyUp"]

    @key_up.setter
    def key_up(self, callback):
        self._events["KeyUp"]._safe_assign(callback)

    @property
    def click(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [Click() -> None]"""
        return self._events["Click"]

    @click.setter
    def click(self, callback):
        self._events["Click"]._safe_assign(callback)

    @property
    def dbl_click(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [DblClick() -> None]"""
        return self._events["DblClick"]

    @dbl_click.setter
    def dbl_click(self, callback):
        self._events["DblClick"]._safe_assign(callback)

    @property
    def mouse_down(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [MouseDown(Button:int, Shift:int, X:int, Y:int) -> None]"""
        return self._events["MouseDown"]

    @mouse_down.setter
    def mouse_down(self, callback):
        self._events["MouseDown"]._safe_assign(callback)

    @property
    def mouse_move(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [MouseMove(Button:int, Shift:int, X:int, Y:int) -> None]"""
        return self._events["MouseMove"]

    @mouse_move.setter
    def mouse_move(self, callback):
        self._events["MouseMove"]._safe_assign(callback)

    @property
    def mouse_up(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [MouseUp(Button:int, Shift:int, X:int, Y:int) -> None]"""
        return self._events["MouseUp"]

    @mouse_up.setter
    def mouse_up(self, callback):
        self._events["MouseUp"]._safe_assign(callback)

    @property
    def ole_drag_drop(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OLEDragDrop(Data:"DataObject", Effect:int, Button:int, Shift:int, X:int, Y:int) -> None]"""
        return self._events["OLEDragDrop"]

    @ole_drag_drop.setter
    def ole_drag_drop(self, callback):
        self._events["OLEDragDrop"]._safe_assign(callback)

    @property
    def mouse_wheel(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [MouseWheel(Button:int, Shift:int, Delta:int, X:int, Y:int) -> None]"""
        return self._events["MouseWheel"]

    @mouse_wheel.setter
    def mouse_wheel(self, callback):
        self._events["MouseWheel"]._safe_assign(callback)


class IUiAxGraphics2DCntrlEventHandler(STKEventSubscriber, IUiAxStockEventHandler):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        IUiAxStockEventHandler.__init__(self)
        if type(interface)==IUnknown:
            impl = IGraphics2DControlEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            raise STKRuntimeError(f"Active X Control events are not available with gRPC.")
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create IUiAxGraphics2DCntrlEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        try:
            IUiAxStockEventHandler.__setattr__(self, attrname, value)
        except:
            if attrname in IUiAxGraphics2DCntrlEventHandler.__dict__ and type(IUiAxGraphics2DCntrlEventHandler.__dict__[attrname]) == property:
                IUiAxGraphics2DCntrlEventHandler.__dict__[attrname].__set__(self, value)
            else:
                raise STKAttributeError(attrname + " is not a recognized event in Graphics2DControlBaseEvents.")


class IUiAxGraphics3DCntrlEventHandler(STKEventSubscriber, IUiAxStockEventHandler):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["_OnObjectEditingStart"]     = _STKEvent()
        self._events["_OnObjectEditingApply"]     = _STKEvent()
        self._events["_OnObjectEditingCancel"]    = _STKEvent()
        self._events["_OnObjectEditingStop"]      = _STKEvent()
        if type(interface)==IUnknown:
            impl = IGraphics3DControlEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            raise STKRuntimeError(f"Active X Control events are not available with gRPC.")
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create IUiAxGraphics3DCntrlEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        try:
            IUiAxStockEventHandler.__setattr__(self, attrname, value)
        except:
            if attrname in IUiAxGraphics3DCntrlEventHandler.__dict__ and type(IUiAxGraphics3DCntrlEventHandler.__dict__[attrname]) == property:
                IUiAxGraphics3DCntrlEventHandler.__dict__[attrname].__set__(self, value)
            else:
                raise STKAttributeError(attrname + " is not a recognized event in Graphics3DControlBaseEvents.")

    @property
    def on_object_editing_start(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnObjectEditingStart(Path:str) -> None]"""
        return self._events["OnObjectEditingStart"]

    @on_object_editing_start.setter
    def on_object_editing_start(self, callback):
        self._events["OnObjectEditingStart"]._safe_assign(callback)

    @property
    def on_object_editing_apply(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnObjectEditingApply(Path:str) -> None]"""
        return self._events["OnObjectEditingApply"]

    @on_object_editing_apply.setter
    def on_object_editing_apply(self, callback):
        self._events["OnObjectEditingApply"]._safe_assign(callback)

    @property
    def on_object_editing_cancel(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnObjectEditingCancel(Path:str) -> None]"""
        return self._events["OnObjectEditingCancel"]

    @on_object_editing_cancel.setter
    def on_object_editing_cancel(self, callback):
        self._events["OnObjectEditingCancel"]._safe_assign(callback)

    @property
    def on_object_editing_stop(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [OnObjectEditingStop(Path:str) -> None]"""
        return self._events["OnObjectEditingStop"]

    @on_object_editing_stop.setter
    def on_object_editing_stop(self, callback):
        self._events["OnObjectEditingStop"]._safe_assign(callback)


################################################################################
#          IAgStkGraphicsSceneEvents
################################################################################

class ISceneEventHandler(STKEventSubscriber):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["Rendering"] = _STKEvent()
        if type(interface)==IUnknown:
            impl = ISceneEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            impl = ISceneEventGrpcHandler(interface, self._events)
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create ISceneEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        if attrname in ISceneEventHandler.__dict__ and type(ISceneEventHandler.__dict__[attrname]) == property:
            ISceneEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in SceneEvents.")

    @property
    def rendering(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [Rendering(Sender:typing.Any, Args:"RenderingEventArgs") -> None]"""
        return self._events["Rendering"]

    @rendering.setter
    def rendering(self, callback):
        self._events["Rendering"]._safe_assign(callback)


################################################################################
#          IAgStkGraphicsKmlGraphicsEvents
################################################################################

class IKmlGraphicsEventHandler(STKEventSubscriber):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["DocumentLoaded"] = _STKEvent()
        if type(interface)==IUnknown:
            impl = IKmlGraphicsEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            impl = IKmlGraphicsEventGrpcHandler(interface, self._events)
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create IKmlGraphicsEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        if attrname in IKmlGraphicsEventHandler.__dict__ and type(IKmlGraphicsEventHandler.__dict__[attrname]) == property:
            IKmlGraphicsEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in KmlGraphicsEvents.")

    @property
    def document_loaded(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [DocumentLoaded(Sender:typing.Any, Args:"KmlDocumentLoadedEventArgs") -> None]"""
        return self._events["DocumentLoaded"]

    @document_loaded.setter
    def document_loaded(self, callback):
        self._events["DocumentLoaded"]._safe_assign(callback)


################################################################################
#          IAgStkGraphicsImageCollectionEvents
################################################################################

class IImageCollectionEventHandler(STKEventSubscriber):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["AddComplete"] = _STKEvent()
        if type(interface)==IUnknown:
            impl = IImageCollectionEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            impl = IImageCollectionEventGrpcHandler(interface, self._events)
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create IImageCollectionEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        if attrname in IImageCollectionEventHandler.__dict__ and type(IImageCollectionEventHandler.__dict__[attrname]) == property:
            IImageCollectionEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in ImageCollectionEvents.")

    @property
    def add_complete(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [AddComplete(Sender:typing.Any, Args:"GlobeImageOverlayAddCompleteEventArgs") -> None]"""
        return self._events["AddComplete"]

    @add_complete.setter
    def add_complete(self, callback):
        self._events["AddComplete"]._safe_assign(callback)


################################################################################
#          IAgStkGraphicsTerrainCollectionEvents
################################################################################

class ITerrainOverlayCollectionEventHandler(STKEventSubscriber):
    def __init__(self, interface):
        self.__dict__["_events"] = {}
        self._events["AddComplete"] = _STKEvent()
        if type(interface)==IUnknown:
            impl = ITerrainOverlayCollectionEventCOMHandler(interface, self._events)
        elif type(interface)==GrpcInterface:
            impl = ITerrainOverlayCollectionEventGrpcHandler(interface, self._events)
        else:
            raise STKRuntimeError(f"Unexpected type {type(interface)}, cannot create ITerrainOverlayCollectionEventHandler.")
        STKEventSubscriber.__init__(self, impl)

    def __del__(self):
        STKEventSubscriber.__del__(self)

    def __setattr__(self, attrname, value):
        if attrname in ITerrainOverlayCollectionEventHandler.__dict__ and type(ITerrainOverlayCollectionEventHandler.__dict__[attrname]) == property:
            ITerrainOverlayCollectionEventHandler.__dict__[attrname].__set__(self, value)
        else:
            raise STKAttributeError(attrname + " is not a recognized event in TerrainOverlayCollectionEvents.")

    @property
    def add_complete(self):
        """Use operator += to register or operator -= to unregister callbacks with the signature [AddComplete(Sender:typing.Any, Args:"TerrainOverlayAddCompleteEventArgs") -> None]"""
        return self._events["AddComplete"]

    @add_complete.setter
    def add_complete(self, callback):
        self._events["AddComplete"]._safe_assign(callback)


# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Used to communicate color information between Python and STK.

Color is the color object type and Colors is the factory for creating Color objects.
"""

import typing

from .exceptions import STKColorError


class Color(object):
    """
    An opaque color representation that can be used with the STK Object Model.

    Examples
    --------
    Get and set a four-channel color for the graphics of an STK graphics primitive:
    >>> from ansys.stk.core.utilities.colors import Colors, ColorRGBA
    >>>
    >>> manager = root.current_scenario.scene_manager
    >>> point = manager.initializers.point_batch_primitive.initialize()
    >>>
    >>> lla_pts = [39.88, -75.25, 0, 38.85, -77.04, 0, 37.37, -121.92, 0]
    >>>
    >>> colors = [Colors.Red, ColorRGBA(Colors.Blue, 127), Colors.from_rgba(0, 255, 0, 127)]
    >>>
    >>> point.set_cartographic_with_colors("Earth", lla_pts, colors)

    Get and set a three-channel color for the graphics of an STK graphics primitive:
    >>> from ansys.stk.core.stkobjects import STKObjectType
    >>> from ansys.stk.core.utilities.colors import Color, Colors
    >>>
    >>> facility = root.current_scenario.children.new(STKObjectType.FACILITY, "facility1")
    >>>
    >>> facility.graphics.color = Colors.Blue
    >>> facility.graphics.color = Color.from_rgb(127, 255, 212)
    >>> (r, g, b) = facility.graphics.color.get_rgb()
    """

    def __init__(self):
        """Construct an object of type Color."""
        self._r = 0
        self._g = 0
        self._b = 0

    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return self._r == other._r and self._g == other._g and self._b == other._b

    def _to_ole_color(self) -> int:
        return self._r + self._g*256 + self._b*256*256

    def _from_ole_color(self, ole:int):
        self._r = ole % 256
        self._g = (ole // 256) % 256
        self._b = (ole // (256*256)) % 256

    @staticmethod
    def _validate_rgb(val):
        if val > 255 or val < 0:
            raise STKColorError("RGB values should be between 0 and 255, inclusive.")
        return val

    def _to_argb(self) -> int:
        alpha = 255 #fully opaque
        return self._b + self._g*256 + self._r*256*256 + alpha*256*256*256

    @classmethod
    def from_rgb(cls, r:int, g:int, b:int) -> "Color":
        """Create a new Color from R, G, B values."""
        c = Color()
        c._r = Color._validate_rgb(r)
        c._g = Color._validate_rgb(g)
        c._b = Color._validate_rgb(b)
        return c

    def get_rgb(self) -> typing.Tuple[int, int, int]:
        """Return the R, G, B representation of this color."""
        return (self._r, self._g, self._b)

class ColorRGBA(object):
    """
    A variably translucent color representation that can be used with certain methods in the STK Object Model.

    Examples
    --------
    Get and set a four-channel color for the graphics of an STK graphics primitive:
    >>> from ansys.stk.core.utilities.colors import Colors, ColorRGBA
    >>>
    >>> manager = root.current_scenario.scene_manager
    >>> point = manager.initializers.point_batch_primitive.initialize()
    >>>
    >>> lla_pts = [39.88, -75.25, 0, 38.85, -77.04, 0, 37.37, -121.92, 0]
    >>>
    >>> colors = [Colors.Red, ColorRGBA(Colors.Blue, 127), Colors.from_rgba(0, 255, 0, 127)]
    >>>
    >>> point.set_cartographic_with_colors("Earth", lla_pts, colors)
    """

    def __init__(self, c:Color, alpha=255):
        """Construct an object of type ColorRGBA."""
        self._color = c
        self._alpha = alpha

    def __eq__(self, other):
        """Check equality of the underlying STK references."""
        return self._color == other._color and self._alpha == other._alpha

    def _to_argb(self) -> int:
        return self._color._b + self._color._g*256 + self._color._r*256*256 + self._alpha*256*256*256

    @property
    def alpha(self) -> float:
        """Gets or sets the ColorRGBA object's value for alpha, which ranges between 0 (fully translucent) and 255 (fully opaque)."""
        return self._alpha

    @alpha.setter
    def alpha(self, value:int) -> None:
        if value >= 0 and value <= 255:
            self._alpha = value
        else:
            raise STKColorError("Alpha value should be between 0 (fully translucent) and 255 (fully opaque), inclusive.")

    @property
    def color(self) -> Color:
        """The Color value that contains R, G, B values."""
        return self._color

class _ColorsImpl(object):
    @staticmethod
    def from_rgb(r:int, g:int, b:int) -> Color:
        return Color.from_rgb(r, g, b)

class Colors(object):
    """
    A factory for creating Color objects that may be used with the STK object model.

    Contains factory methods and named colors.

    Examples
    --------
    Get and set a four-channel color for the graphics of an STK graphics primitive:
    >>> from ansys.stk.core.utilities.colors import Colors, ColorRGBA
    >>>
    >>> manager = root.current_scenario.scene_manager
    >>> point = manager.initializers.point_batch_primitive.initialize()
    >>>
    >>> lla_pts = [39.88, -75.25, 0, 38.85, -77.04, 0, 37.37, -121.92, 0]
    >>>
    >>> colors = [Colors.Red, ColorRGBA(Colors.Blue, 127), Colors.from_rgba(0, 255, 0, 127)]
    >>>
    >>> point.set_cartographic_with_colors("Earth", lla_pts, colors)

    Get and set a three-channel color for the graphics of an STK graphics primitive:
    >>> from ansys.stk.core.stkobjects import STKObjectType
    >>> from ansys.stk.core.utilities.colors import Color, Colors
    >>>
    >>> facility = root.current_scenario.children.new(STKObjectType.FACILITY, "facility1")
    >>>
    >>> facility.graphics.color = Colors.Blue
    >>> facility.graphics.color = Color.from_rgb(127, 255, 212)
    >>> (r, g, b) = facility.graphics.color.get_rgb()
    """

    @staticmethod
    def from_rgb(r:int, g:int, b:int) -> Color:
        """Create a new Color from R, G, B values in the range [0, 255]."""
        return _ColorsImpl.from_rgb(r, g, b)

    @staticmethod
    def from_rgba(r:int, g:int, b:int, a:int) -> ColorRGBA:
        """Create a new Color from R, G, B, A values in the range [0, 255]."""
        c = _ColorsImpl.from_rgb(r, g, b)
        return ColorRGBA(c, a)

    @staticmethod
    def from_argb(*args):
        """Create a new Color from an arbitrary number of values in the range [0, 255], inferred from the arguments provided."""
        if len(args) == 1: # argb
            return _ColorsImpl.from_rgb((args[0] & 0xFF0000) >> 16, (args[0] & 0x00FF00) >> 8, args[0] & 0xFF)
        elif len(args) == 3: # r, g, b
            return _ColorsImpl.from_rgb(args[0], args[1], args[2])
        elif len(args) == 4: # a, r, g, b
            return _ColorsImpl.from_rgb(args[1], args[2], args[3])
        else:
            raise STKColorError('unsupported color conversion')

    AliceBlue            = _ColorsImpl.from_rgb(240,  248,  255)
    AntiqueWhite         = _ColorsImpl.from_rgb(250,  235,  215)
    Aqua                 = _ColorsImpl.from_rgb(0,    255,  255)
    Aquamarine           = _ColorsImpl.from_rgb(127,  255,  212)
    Azure                = _ColorsImpl.from_rgb(240,  255,  255)
    Beige                = _ColorsImpl.from_rgb(245,  245,  220)
    Bisque               = _ColorsImpl.from_rgb(255,  228,  196)
    Black                = _ColorsImpl.from_rgb(0,    0,    0  )
    BlanchedAlmond       = _ColorsImpl.from_rgb(255,  235,  205)
    Blue                 = _ColorsImpl.from_rgb(0,    0,    255)
    BlueViolet           = _ColorsImpl.from_rgb(138,  43,   226)
    Brown                = _ColorsImpl.from_rgb(165,  42,   42 )
    Burlywood            = _ColorsImpl.from_rgb(222,  184,  135)
    CadetBlue            = _ColorsImpl.from_rgb(95,   158,  160)
    Chartreuse           = _ColorsImpl.from_rgb(127,  255,  0  )
    Chocolate            = _ColorsImpl.from_rgb(210,  105,  30 )
    Coral                = _ColorsImpl.from_rgb(255,  127,  80 )
    CornflowerBlue       = _ColorsImpl.from_rgb(100,  149,  237)
    Cornsilk             = _ColorsImpl.from_rgb(255,  248,  220)
    Crimson              = _ColorsImpl.from_rgb(220,  20,   60 )
    Cyan                 = _ColorsImpl.from_rgb(0,    255,  255)
    DarkBlue             = _ColorsImpl.from_rgb(0,    0,    139)
    DarkCyan             = _ColorsImpl.from_rgb(0,    139,  139)
    DarkGoldenrod        = _ColorsImpl.from_rgb(184,  134,  11 )
    DarkGray             = _ColorsImpl.from_rgb(169,  169,  169)
    DarkGreen            = _ColorsImpl.from_rgb(0,    100,  0  )
    DarkGrey             = _ColorsImpl.from_rgb(169,  169,  169)
    DarkKhaki            = _ColorsImpl.from_rgb(189,  183,  107)
    DarkMagenta          = _ColorsImpl.from_rgb(139,  0,    139)
    DarkOliveGreen       = _ColorsImpl.from_rgb(85,   107,  47 )
    DarkOrange           = _ColorsImpl.from_rgb(255,  140,  0  )
    DarkOrchid           = _ColorsImpl.from_rgb(153,  50,   204)
    DarkRed              = _ColorsImpl.from_rgb(139,  0,    0  )
    DarkSalmon           = _ColorsImpl.from_rgb(233,  150,  122)
    DarkSeaGreen         = _ColorsImpl.from_rgb(143,  188,  143)
    DarkSlateBlue        = _ColorsImpl.from_rgb(72,   61,   139)
    DarkSlateGray        = _ColorsImpl.from_rgb(47,   79,   79 )
    DarkSlateGrey        = _ColorsImpl.from_rgb(47,   79,   79 )
    DarkTurquoise        = _ColorsImpl.from_rgb(0,    206,  209)
    DarkViolet           = _ColorsImpl.from_rgb(148,  0,    211)
    DeepPink             = _ColorsImpl.from_rgb(255,  20,   147)
    DeepSkyBlue          = _ColorsImpl.from_rgb(0,    191,  255)
    DimGray              = _ColorsImpl.from_rgb(105,  105,  105)
    DimGrey              = _ColorsImpl.from_rgb(105,  105,  105)
    DodgerBlue           = _ColorsImpl.from_rgb(30,   144,  255)
    FireBrick            = _ColorsImpl.from_rgb(178,  34,   34 )
    FloralWhite          = _ColorsImpl.from_rgb(255,  250,  240)
    ForestGreen          = _ColorsImpl.from_rgb(34,   139,  34 )
    Fuchsia              = _ColorsImpl.from_rgb(255,  0,    255)
    Gainsboro            = _ColorsImpl.from_rgb(220,  220,  220)
    GhostWhite           = _ColorsImpl.from_rgb(248,  248,  255)
    Gold                 = _ColorsImpl.from_rgb(255,  215,  0  )
    Goldenrod            = _ColorsImpl.from_rgb(218,  165,  32 )
    Gray                 = _ColorsImpl.from_rgb(128,  128,  128)
    Green                = _ColorsImpl.from_rgb(0,    128,  0  )
    GreenYellow          = _ColorsImpl.from_rgb(173,  255,  47 )
    Grey                 = _ColorsImpl.from_rgb(128,  128,  128)
    Honeydew             = _ColorsImpl.from_rgb(240,  255,  240)
    HotPink              = _ColorsImpl.from_rgb(255,  105,  180)
    IndianRed            = _ColorsImpl.from_rgb(205,  92,   92 )
    Indigo               = _ColorsImpl.from_rgb(75,   0,    130)
    Ivory                = _ColorsImpl.from_rgb(255,  255,  240)
    Khaki                = _ColorsImpl.from_rgb(240,  230,  140)
    Lavender             = _ColorsImpl.from_rgb(230,  230,  250)
    LavenderBlush        = _ColorsImpl.from_rgb(255,  240,  245)
    Lawngreen            = _ColorsImpl.from_rgb(124,  252,  0  )
    LemonChiffon         = _ColorsImpl.from_rgb(255,  250,  205)
    LightBlue            = _ColorsImpl.from_rgb(173,  216,  230)
    LightCoral           = _ColorsImpl.from_rgb(240,  128,  128)
    LightCyan            = _ColorsImpl.from_rgb(224,  255,  255)
    LightGoldenrodYellow = _ColorsImpl.from_rgb(250,  250,  210)
    LightGray            = _ColorsImpl.from_rgb(211,  211,  211)
    LightGreen           = _ColorsImpl.from_rgb(144,  238,  144)
    LightGrey            = _ColorsImpl.from_rgb(211,  211,  211)
    LightPink            = _ColorsImpl.from_rgb(255,  182,  193)
    LightSalmon          = _ColorsImpl.from_rgb(255,  160,  122)
    LightSeaGreen        = _ColorsImpl.from_rgb(32,   178,  170)
    LightSkyBlue         = _ColorsImpl.from_rgb(135,  206,  250)
    LightSlateGray       = _ColorsImpl.from_rgb(119,  136,  153)
    LightSlateGrey       = _ColorsImpl.from_rgb(119,  136,  153)
    LightSteelBlue       = _ColorsImpl.from_rgb(176,  196,  222)
    LightYellow          = _ColorsImpl.from_rgb(255,  255,  224)
    Lime                 = _ColorsImpl.from_rgb(0,    255,  0  )
    LimeGreen            = _ColorsImpl.from_rgb(50,   205,  50 )
    Linen                = _ColorsImpl.from_rgb(250,  240,  230)
    Magenta              = _ColorsImpl.from_rgb(255,  0,    255)
    Maroon               = _ColorsImpl.from_rgb(128,  0,    0  )
    MediumAquamarine     = _ColorsImpl.from_rgb(102,  205,  170)
    MediumBlue           = _ColorsImpl.from_rgb(0,    0,    205)
    MediumOrchid         = _ColorsImpl.from_rgb(186,  85,   211)
    MediumPurple         = _ColorsImpl.from_rgb(147,  112,  219)
    MediumSeaGreen       = _ColorsImpl.from_rgb(60,   179,  113)
    MediumSlateBlue      = _ColorsImpl.from_rgb(123,  104,  238)
    MediumSpringGreen    = _ColorsImpl.from_rgb(0,    250,  154)
    MediumTurquoise      = _ColorsImpl.from_rgb(72,   209,  204)
    MediumVioletRed      = _ColorsImpl.from_rgb(199,  21,   133)
    MidnightBlue         = _ColorsImpl.from_rgb(25,   25,   112)
    MintCream            = _ColorsImpl.from_rgb(245,  255,  250)
    MistyRose            = _ColorsImpl.from_rgb(255,  228,  225)
    Moccasin             = _ColorsImpl.from_rgb(255,  228,  181)
    NavajoWhite          = _ColorsImpl.from_rgb(255,  222,  173)
    Navy                 = _ColorsImpl.from_rgb(0,    0,    128)
    OldLace              = _ColorsImpl.from_rgb(253,  245,  230)
    Olive                = _ColorsImpl.from_rgb(128,  128,  0  )
    OliveDrab            = _ColorsImpl.from_rgb(107,  142,  35 )
    Orange               = _ColorsImpl.from_rgb(255,  165,  0  )
    OrangeRed            = _ColorsImpl.from_rgb(255,  69,   0  )
    Orchid               = _ColorsImpl.from_rgb(218,  112,  214)
    PaleGoldenrod        = _ColorsImpl.from_rgb(238,  232,  170)
    PaleGreen            = _ColorsImpl.from_rgb(152,  251,  152)
    PaleTurquoise        = _ColorsImpl.from_rgb(175,  238,  238)
    PaleVioletRed        = _ColorsImpl.from_rgb(219,  112,  147)
    PapayaWhip           = _ColorsImpl.from_rgb(255,  239,  213)
    PeachPuff            = _ColorsImpl.from_rgb(255,  218,  185)
    Peru                 = _ColorsImpl.from_rgb(205,  133,  63 )
    Pink                 = _ColorsImpl.from_rgb(255,  192,  203)
    Plum                 = _ColorsImpl.from_rgb(221,  160,  221)
    PowderBlue           = _ColorsImpl.from_rgb(176,  224,  230)
    Purple               = _ColorsImpl.from_rgb(128,  0,    128)
    Red                  = _ColorsImpl.from_rgb(255,  0,    0  )
    RosyBrown            = _ColorsImpl.from_rgb(188,  143,  143)
    RoyalBlue            = _ColorsImpl.from_rgb(65,   105,  225)
    SaddleBrown          = _ColorsImpl.from_rgb(139,  69,   19 )
    Salmon               = _ColorsImpl.from_rgb(250,  128,  114)
    SandyBrown           = _ColorsImpl.from_rgb(244,  164,  96 )
    SeaGreen             = _ColorsImpl.from_rgb(46,   139,  87 )
    Seashell             = _ColorsImpl.from_rgb(255,  245,  238)
    Sienna               = _ColorsImpl.from_rgb(160,  82,   45 )
    Silver               = _ColorsImpl.from_rgb(192,  192,  192)
    SkyBlue              = _ColorsImpl.from_rgb(135,  206,  235)
    SlateBlue            = _ColorsImpl.from_rgb(106,  90,   205)
    SlateGray            = _ColorsImpl.from_rgb(112,  128,  144)
    SlateGrey            = _ColorsImpl.from_rgb(112,  128,  144)
    Snow                 = _ColorsImpl.from_rgb(255,  250,  250)
    SpringGreen          = _ColorsImpl.from_rgb(0,    255,  127)
    SteelBlue            = _ColorsImpl.from_rgb(70,   130,  180)
    Tan                  = _ColorsImpl.from_rgb(210,  180,  140)
    Teal                 = _ColorsImpl.from_rgb(0,    128,  128)
    Thistle              = _ColorsImpl.from_rgb(216,  191,  216)
    Tomato               = _ColorsImpl.from_rgb(255,  99,   71 )
    Turquoise            = _ColorsImpl.from_rgb(64,   224,  208)
    Violet               = _ColorsImpl.from_rgb(238,  130,  238)
    Wheat                = _ColorsImpl.from_rgb(245,  222,  179)
    White                = _ColorsImpl.from_rgb(255,  255,  255)
    WhiteSmoke           = _ColorsImpl.from_rgb(245,  245,  245)
    Yellow               = _ColorsImpl.from_rgb(255,  255,  0  )
    YellowGreen          = _ColorsImpl.from_rgb(154,  205,  50 )
